<?php

namespace HCM\FPA\ObrigacaoLegal\Dirf;

use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroDIRF;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroIDREC;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroBPFDEC;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroRTRT;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroRTPO;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroRTIRF;
use HCM\Util\DateUtil;
use HCM\Util\MsgCalculo;
use HCM\Util\Repositories;

class Dirf2021 {

	public function __construct($entityManager, $hcmParameters, $objMagnetico, $nrRegimeApur){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdOperador();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_nrTipoModalidadeCalculo = $hcmParameters->getTipoModalidadeCalculo();
        
        $this->_calculoFolha = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        $this->_itemCalculo = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        $this->_evento = new FpaeventomEntity($entityManager, $hcmParameters);
        
        $this->_objMagnetico = $objMagnetico;
        $this->_nrRegimeApur = $nrRegimeApur;

        $this->_registroIDREC = new RegistroIDREC();
        $this->_registroBPFDEC = new RegistroBPFDEC();
        $this->_registroRTRT = new RegistroRTRT();
        $this->_registroRTPO = new RegistroRTPO();
        $this->_registroRTIRF = new RegistroRTIRF();
    }

    public function geraDIRF($vinculo){
        $this->_vinculo = $vinculo;
        $this->_dtRescisao = $this->_vinculo->getDtrescisaovinc();
        $this->_codMovimentacao = null;
        
        $this->_pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaAtual(
            $this->_nrOrg, $vinculo->getNrpessoa(), $this->_mesCompetencia
        );

        $this->_vrTotal = 0;
        $this->_vrTotal13 = 0;
        $this->_dsComplemento = null;

        $this->_calculaDIRF();

        if (count($this->_dadoVinculo) > 0) {
            $this->_preparaRegistroIDREC();
            
            $this->_preparaRegistroBPFDEC();

            $this->_preparaRegistroRTRT();
            $this->_preparaRegistroRTPO();
            $this->_preparaRegistroRTIRF();

            $this->_objMagnetico->fimAddRegVinculo();
        }
    }

    private function _calculaDIRF()
    {
        $this->_dadoVinculo = array();

        //for ($mes = 1; $mes <= 13; $mes++) {
            $this->_calculaRTRT();
            $this->_calculaRTPO();
            $this->_calculaRTIRF();
        //}

        //var_dump($this->_dadoVinculo);
    }

    private function _consolidaDadosVinculo($referencia, $mes, $valor)
    {
        $this->_dadoVinculo[$referencia][$mes] += $valor;
    }

    private function _calculaRTRT()
    {
        //$dtmescompetenc = DateUtil::setMonth($this->_mesCompetencia, 4);
        
        $bases = $this->_itemCalculo->retornaItensPorVinculoDirf($this->_nrOrg, $this->_vinculo->getNrvinculom(), $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $this->_nrRegimeApur, 'RTRT');
        //$bases = $this->_calculaValorItemGrupo(RegistroDIRF::ITEM_RTRT, $dtmescompetenc);

        //foreach ($bases as $mes => $valor) {
        foreach ($bases as $evento) {

            $ano = substr($evento['MES'],0,4);
            $mes = (int) substr($evento['MES'],4,2);

            //var_dump($evento['MES'].', '.$mes.', '.$evento['VREVENTOFOLHA']);
            if ($this->_mesCompetencia->format('Y') == $ano) {

                //$this->_consolidaDadosVinculo(RegistroDIRF::ITEM_RTRT, $mes, $valor);
                $this->_consolidaDadosVinculo(RegistroDIRF::ITEM_RTRT, $mes, $evento['VREVENTOFOLHA']);

                if ($mes == 13) {
                    $this->_vrTotal13 += $evento['VREVENTOFOLHA'];
                } else {
                    $this->_vrTotal += $evento['VREVENTOFOLHA'];
                }

            }
        }
    }

    private function _calculaRTPO()
    {
        $bases = $this->_itemCalculo->retornaItensPorVinculoDirf($this->_nrOrg, $this->_vinculo->getNrvinculom(), $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $this->_nrRegimeApur, 'RTPO');

        foreach ($bases as $evento) {

            $ano = substr($evento['MES'],0,4);
            $mes = (int) substr($evento['MES'],4,2);

            if ($this->_mesCompetencia->format('Y') == $ano) {
                $this->_consolidaDadosVinculo(RegistroDIRF::ITEM_RTPO, $mes, $evento['VREVENTOFOLHA']);
            }
        }
    }

    private function _calculaRTIRF()
    {
        $bases = $this->_itemCalculo->retornaItensPorVinculoDirf($this->_nrOrg, $this->_vinculo->getNrvinculom(), $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $this->_nrRegimeApur, 'RTIRF');

        foreach ($bases as $evento) {

            $ano = substr($evento['MES'],0,4);
            $mes = (int) substr($evento['MES'],4,2);

            if ($this->_mesCompetencia->format('Y') == $ano) {
                $this->_consolidaDadosVinculo(RegistroDIRF::ITEM_RTIRF, $mes, $evento['VREVENTOFOLHA']);
            }
        }
    }

    private function _preparaRegistroIDREC(){

        $cdReceita = '';

        if ($this->_vinculo->getNrtipovinculom() == 2) {
            $cdReceita = '0588';
        } else {
            $vinculoempreg = $this->entityManager->getRepository(Repositories::GPE_VINCULOEMPREG)->findOneBy(
                array('nrvinculoempreg' => $this->_vinculo->getNrvinculoempreg())
            );
            $cdReceita = $vinculoempreg->getCodreceita();
        }

        $this->_registroIDREC->setCdreceita($cdReceita);

        $this->_registroIDREC->geraRegistroIDREC();

        $this->_idrec = $this->_registroIDREC->getLinhaRegistro();

        $this->_registroIDREC->setLinhaRegistro();

    }

    private function _preparaRegistroBPFDEC(){

        $this->_registroBPFDEC->setCpf($this->_pessoa->historico->getNrcpfpessoa());
        $this->_registroBPFDEC->setNome($this->_vinculo->getNmvinculom());
        $this->_registroBPFDEC->setDtmolestia(null);
        $this->_registroBPFDEC->setIdalimentado('');
        $this->_registroBPFDEC->setIdprevidencia('');

        $this->_registroBPFDEC->geraRegistroBPFDEC();

        $this->_bpfdec = $this->_registroBPFDEC->getLinhaRegistro();

        $this->_registroBPFDEC->setLinhaRegistro();

    }

    private function _preparaRegistroRTRT(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTRT])) {
            $this->_registroRTRT->geraRegistroRTRT($this->_dadoVinculo[RegistroDIRF::ITEM_RTRT]);
            $this->_addRegVinculo('RTRT', $this->_registroRTRT->getLinhaRegistro());

            $this->_registroRTRT->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRTPO(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTPO])) {
            $this->_registroRTPO->geraRegistroRTPO($this->_dadoVinculo[RegistroDIRF::ITEM_RTPO]);
            $this->_addRegVinculo('RTPO', $this->_registroRTPO->getLinhaRegistro());

            $this->_registroRTPO->setLinhaRegistro();
        }

    }

    private function _preparaRegistroRTIRF(){

        if (isset($this->_dadoVinculo[RegistroDIRF::ITEM_RTIRF])) {
            $this->_registroRTIRF->geraRegistroRTIRF($this->_dadoVinculo[RegistroDIRF::ITEM_RTIRF]);
            $this->_addRegVinculo('RTIRF', $this->_registroRTIRF->getLinhaRegistro());

            $this->_registroRTIRF->setLinhaRegistro();
        }

    }

    private function _addRegVinculo($nmRegistro, $linhaRegistro){
        $this->_objMagnetico->addRegVinculo(
            $this->_pessoa->historico->getNrcpfpessoa(), $this->_pessoa->historico->getNmpessoa(), null, $nmRegistro, $this->_registroIDREC->getCdreceita(), $this->_vrTotal, $this->_vrTotal13, 
            $this->_idrec, $this->_bpfdec, $linhaRegistro, $this->_dsComplemento, $this->_vinculo->historico->getNrestrutlegal()
        );
    }

}