<?php

namespace HCM\FPA\ObrigacaoLegal\Dirf;

use HCM\Geral\Entity\GpevinculomEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpadirfmagnetEntity;
use HCM\FPA\ObrigacaoLegal\Dirf\Dirf2021;
use HCM\FPA\ObrigacaoLegal\Dirf\QueryDirf;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroDIRF;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroRESPO;
use HCM\FPA\ObrigacaoLegal\Dirf\Registros\RegistroDECPJ;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\MsgCalculo;
use HCM\Util\Repositories;

class PercorreDirf {

    private $_objMagnetico;

	public function __construct($entityManager, $hcmParameters, $dataSet){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdOperador();
        $this->_dtCompetencia = $hcmParameters->getDtCompetencia();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);

        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_calculoFolha = new FpacalculofolhaEntity($entityManager, $hcmParameters);

        $this->_inicializaVariaveis($dataSet);

        $this->_objMagnetico = new FpadirfmagnetEntity($this->entityManager, $this->hcmParameters, $this->_nrOrg, $this->_mesCompetencia, $this->_nrInscEstrutResp);

        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch (strtoupper($tp_processamento)){
            case FPA::PROC_COMPETENCIA_MENSAL :
                $this->_retrocesso = false;
                break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true;
                break;
            default:
                throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');
        }
    }

    public function _inicializaVariaveis($dataSet) {

        $this->_registroDIRF = new RegistroDIRF();
        $this->_registroRESPO = new RegistroRESPO();
        $this->_registroDECPJ = new RegistroDECPJ();

        $this->queryDirf = new QueryDirf($this->entityManager,$this->hcmParameters);

        $this->_buscaDadosFormulario($dataSet);

        $estruturaResp = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstrutura($this->_nrOrg, $this->_nrEstrResponsa, $this->_mesCompetencia);
        $this->_nrInscEstrutResp = $estruturaResp->historico->getCdcnpjestrut();
        
        // Calculo
        $this->_nrCalculoFolha = $this->_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias(
            $this->_nrOrg, 1, $this->_mesCompetencia, $this->hcmParameters->getTipoModalidadeCalculo()
        );
            
        // Criado valor de referencia apenas para parametrização de qual versão da DIRF usar: 1-versão utilizada até a dirf 2019, 2-versão com tabela temporária ou 3-versão com tabela de lançamentos de vínculos 
        $this->versaoQuery = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->
            buscavalor($this->_nrOrg, $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getDtCompetencia(), 'VERSAO_QUERY_DIRF');
            
        // Criado valor de referencia para executar as queries da dirf por vínculo para reduzir custo do banco
        $this->versaoVinculo = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->
            buscavalor($this->_nrOrg, $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getDtCompetencia(), 'VERSAO_DIRF_VINCULO');

    }
	
	//Função que carrega os dados do formulário. Na falha esta seta o nrOrg para -1 indicando a falha
    private function _buscaDadosFormulario($dataSet){
        $formObj = $this->entityManager->getRepository(Repositories::FPA_FORMDIRF)->findOneBy(array("nrorg" => $dataSet['NRORG'], "nrformdirf"=>$dataSet['NRFORMDIRF']));
		//Dados do formulário
        if(is_object($formObj)) {   //Somente grava os dados se encontrou o formulário segundo a Org e seu Nro
            $this->_nrForm         = $formObj->getNrformdirf();
            $this->_nmForm         = $formObj->getNmformdirf();
            $this->_idProcessaVinc = $formObj->getIdprocvinculo();
            $this->_idGeraMagnetic = $formObj->getIdgerameiomag();
            $this->_idExtinsaoEmpr = $formObj->getIdexcempresa();
            $this->_idTipoDeclarac = $formObj->getIdtipdeclaracao();
            $this->_idNaturezaDecl = $formObj->getIdnaturezadirf();
            $this->_dsOrdem        = $formObj->getDsordem();
            $this->_dsInicio       = $formObj->getNomeinicial();
            $this->_dsFim          = $formObj->getNomefinal();
			$this->_nomeInicial    = $formObj->getNomeinicial(); /*pck add 24/01/19 inicio*/
            $this->_nomeFinal      = $formObj->getNomefinal();
            $this->_dataInicial    = $formObj->getDtinicial();
            $this->_dataFinal      = $formObj->getDtfinal();
            $this->_nrEstrutuLegal = $formObj->getNrestrutlegal();
            $this->_nrEstrutuGeren = $formObj->getNrestrutgeren();
            $this->_nrMatriInicial = $formObj->getNrmatricinicial();
            $this->_nrMatricuFinal = $formObj->getNrmatricfinal(); /*pck add 24/01/19 fim*/
            $this->_nrPessResponsa = $formObj->getNrpessoarespons();
            $this->_nrEstrResponsa = $formObj->getNrestrutrespons();
            $this->_nrGrupoInciden = $formObj->getNrgrpincide();
            $this->_nrGrupoIncSaud = $formObj->getNrgrpincidsaud();
            $this->_nrRecibo       = $formObj->getNrregistro();
            $this->_idTpInscriResp = $formObj->getIdtpinscrespons();
            $this->_idSocioOstensi = $formObj->getIdsociostens();
            $this->_idDepositJudic = $formObj->getIddepojudic();
            $this->_idInstitInvest = $formObj->getIdinstinvest();
            $this->_idPagRendiExte = $formObj->getIdpagrendext();
            $this->_idPagPlanoSaud = $formObj->getIdpagplansaud();
            $this->_idPagRelacCopa = $formObj->getIdpagrelcopa();
            //$this->_idPagRelacOlimp = $formObj->getIdpagrelolimp();    //DIRF 2019
            $this->_idPagRelacOlimp = 'N';                               //DIRF 2019 - N – Não é fundação pública de direito privado
            //$this->_idPagSituaEspe = $formObj->getIdpagsitespe();      //DIRF 2019
            $this->_idPagSituaEspe = 'N';                                //DIRF 2019 - N – Não é declaração de situação especial
            $this->_dtEventoEspeci = $formObj->getDteventespec();
            $this->_anoCalendario  = $formObj->getNranocalen();
            $this->_anoReferencia  = $formObj->getNranoref();
            $this->_tdVinculos = $formObj->getIdtdvinculos();
            $this->_idVincRetencao = $formObj->getIdretvinculos();
            $this->_idVincRendimento = $formObj->getIdrendvinculos();
            $this->_vrMinRendimento = $formObj->getVrreferencia();
            $this->_idVincProLabore = $formObj->getIdprovinculos();
            $this->params = array();
            $this->params["inicio"] = $this->dsInicio;
            if($this->dsFim) {
                $this->params["fim"] = $this->dsFim;
            }
        } else {				//Formulário não encontrado
            $this->nrorg  = -1;	//indica que não carregou os dados
            $this->msgErro = 'Formulário não carregado. O número do formulário '.$dataSet['NRFORMDIRF'].' da organização número '.$dataSet['NRORG'].' não foi encontrado.'.Dirf::ql_msg;
        }

    }
    
    public function percorre(){
        if ($this->_retrocesso === false) {
            try{
                $registros = $this->_objMagnetico->qdeRegistros();
                if ($registros == 0) {
                    $resultado = $this->_geraDIRF();
                    return $resultado;
                } else {
                    throw new \Exception('Já existe magnético da DIRF para o mês de competência, favor retroceder.');
                }
            }catch (\Exception $e) {
                throw new \Exception($e->getMessage(), $e->getCode());
            }
        } else {
            try {
                $this->_sefip->retroceder();
                return array("RETROCESSO" => true);
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage(), $e->getCode());
            }
        }
    }

    private function _geraDIRF(){

        $this->_msg_idabort = false;    
        $this->_msg_idstatus = MsgCalculo::ID_PENDENTE;
        $nrcalculomsg = $this->_nrCalculoFolha;

        if ($this->_idProcessaVinc == 'S') {

            if ($this->_idGeraMagnetic == 'S') {
                $this->_objMagnetico->addRegCabecal($this->_nrInscEstrutResp, null, $this->_nrPessResponsa, 'S');
            }

            $this->_preparaRegistroDIRF();
            $this->_preparaRegistroRESPO();
            $this->_preparaRegistroDECPJ();

            $vinculos = $this->_vinculom->retornaVinculosDirf($this->_nrOrg, $this->_mesCompetencia,
                $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, $this->_tdVinculos, $this->_idVincRetencao, $this->_idVincRendimento, $this->_vrMinRendimento, $this->_idVincProLabore
            );

            $this->_dirf  = new Dirf2021(
                $this->entityManager, $this->hcmParameters, $this->_objMagnetico, $this->_nrRegimeApur
            );

            foreach ($vinculos as $vinculo) {
                $processamento = MsgCalculo::getInstance();
                $processamento->setDados($nrcalculomsg, $vinculo->getNrvinculom(), $this->_nrOrg, 'DIRF');

                $this->_dirf->geraDIRF($vinculo, $this->_nrTpHierarquiaOrg, $this->_nrTpHierarqTomador, $this->_tpInscHierarq, $this->_tpInscHierarqTomador);
            }

        }
        
        if (($this->_msg_idabort == false) && ($this->_msg_idstatus !== MsgCalculo::ID_COM_ERRO)) {
            if ($this->_idGeraMagnetic == 'S') {
                $arquivo = $this->_geraMagnetico();
            }
        }
        
        return array(
            'ARQUIVO' => $arquivo,
            'CALCULO' => $nrcalculomsg,
            'ERRO' => (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) || ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)),
            'ABORT' => $this->_msg_idabort
        );
    }

    private function _geraMagnetico(){

        $nomeArquivo = "DIRF_" . $this->_nrInscEstrutResp . ".txt";

        $this->_registroDIRF->geraRegistroDIRF();
        FPA::escreveArquivo($nomeArquivo, $this->_registroDIRF->getLinhaRegistro().RegistroDIRF::QUEBRA_LINHA_CR_LF, false, true);

        $this->_registroRESPO->geraRegistroRESPO();
        FPA::escreveArquivo($nomeArquivo, $this->_registroRESPO->getLinhaRegistro().RegistroDIRF::QUEBRA_LINHA_CR_LF);

        $this->_registroDECPJ->geraRegistroDECPJ();
        FPA::escreveArquivo($nomeArquivo, $this->_registroDECPJ->getLinhaRegistro().RegistroDIRF::QUEBRA_LINHA_CR_LF);
        
        $idrecAnt = -1;
        $registros = $this->_objMagnetico->retornaRegistros();
        foreach ($registros as $item) {
            $idrec = substr($item->getDslinharegistro(),0,strpos($item->getDslinharegistro(),"\nBPFDEC"));
            $registro = substr($item->getDslinharegistro(),strpos($item->getDslinharegistro(),"\nBPFDEC")+1,strlen($item->getDslinharegistro()));

            if ($idrec != $idrecAnt) {
                $idrecAnt = $idrec;
                FPA::escreveArquivo($nomeArquivo, $idrec.RegistroDIRF::QUEBRA_LINHA_CR_LF);
            }

            FPA::escreveArquivo($nomeArquivo, $registro.RegistroDIRF::QUEBRA_LINHA_CR_LF);
        }

        FPA::escreveArquivo($nomeArquivo, $this->_fimDirf().RegistroDIRF::QUEBRA_LINHA_CR_LF);
        
        return $nomeArquivo;
    }

    private function _preparaRegistroDIRF(){

        if(is_null($this->_anoReferencia) || $this->_anoReferencia == ''){
			$this->msgErro .= "Está faltando o ano de referência.".Dirf::ql_msg;
        }

        if(is_null($this->_anoCalendario) || $this->_anoCalendario == ''){
			$this->msgErro .= "Está faltando o ano de calendário.".Dirf::ql_msg;
        }
        
        if($this->_idTipoDeclarac != 'S' && $this->_idTipoDeclarac != 'N' ){
			$this->msgErro .= "Está faltando a indicação do tipo de declaração.".Dirf::ql_msg;
        }

        $anoReferencia  = str_pad(substr($this->_anoReferencia,  0,  4),  4,' '); //Garante o tamanho do campo em 4
        $anoCalendario  = str_pad(substr($this->_anoCalendario,  0,  4),  4,' '); //Garante o tamanho do campo em 4
        $idTipoDeclarac = str_pad(substr($this->_idTipoDeclarac, 0,  1),  1,' '); //Garante o tamanho do campo em 1
        $auxNrRecibo    = $this->_nrRecibo == null ? '' : str_pad(substr($this->_nrRecibo, 0, 12), 12, '0', STR_PAD_LEFT);

        if($anoReferencia >= '2022'){
            $ideEstrutLeiaute = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
                $this->_nrOrg, $this->_nrOrg, $this->_mesCompetencia->format(DateUtil::FORMATO_BRASILEIRO), RegistroDIRF::IDE_LEIAUTE_DIRF
            );
        } else {
            switch ($anoReferencia) {
                case '2014':
                    $idEstrutLeiaute = 'F8UCL6S|'; break;
                case '2015':
                    $idEstrutLeiaute = 'M1LB5V2|'; break;
                case '2016':
                    $idEstrutLeiaute = 'L35QJS2|'; break;
                case '2017':
                    $idEstrutLeiaute = 'P49VS72|'; break;
                case '2018':
                    $idEstrutLeiaute = 'Q84FV63|'; break;	
                case '2019':
                    $idEstrutLeiaute = 'T17BS45|'; break;
                case '2020':
                    $idEstrutLeiaute = 'AT65HD8|'; break;
                case '2021':
                    $idEstrutLeiaute = 'VR4QLM8|'; break;
                default:
                    $idEstrutLeiaute = '|'; break;
            }
        }
        
        $this->_registroDIRF->setAnoReferencia($anoReferencia);
        $this->_registroDIRF->setAnoCalendario($anoCalendario);
        $this->_registroDIRF->setIdTipoDeclarac($idTipoDeclarac);
        $this->_registroDIRF->setNrRecibo($auxNrRecibo);
        $this->_registroDIRF->setIdeLeiaute($ideEstrutLeiaute);

    }

    private function _preparaRegistroRESPO(){

        try{
            $params = array();
            $params['nrorg']            = $this->_nrOrg;
            $params['nrpessresponsa']   = $this->_nrPessResponsa;
            $params['dtcompetencia']    = $this->_dtCompetencia;
            $sqlResult = $this->queryDirf->get_respo($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo RESPO, item 3.2.');
        }

        foreach($sqlResult as $val){
            if($val['NRCPF']      == '|'){
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem CPF.");
				$this->msgErro .= "Responsável pela DIRF sem CPF.".Dirf::ql_msg;
            }
            
            if($val['NMNOMERESP'] == '|'){
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem nome.");
				$this->msgErro .= "Responsável pela DIRF sem nome.".Dirf::ql_msg;
            }
            
            if($val['NRDDD']      == '|'){
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem Nº de DDD.");
				$this->msgErro .= "Responsável pela DIRF sem Nº de DDD.".Dirf::ql_msg;
            }
            
            if($val['NRTELEFONE'] == '|'){
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável pela DIRF sem Nº de telefone.");
				$this->msgErro .= "Responsável pela DIRF sem Nº de telefone.".Dirf::ql_msg;
            }

            $this->_registroRESPO->setCpf(str_replace('|','',$val['NRCPF']));
            $this->_registroRESPO->setNome(str_replace('|','',$val['NMNOMERESP']));
            $this->_registroRESPO->setDdd(str_replace('|','',$val['NRDDD']));
            $this->_registroRESPO->setTelefone(str_replace('|','',$val['NRTELEFONE']));
            $this->_registroRESPO->setRamal(str_replace('|','',$val['NRRAMAL']));
            $this->_registroRESPO->setFax(str_replace('|','',$val['NRFAX']));
            $this->_registroRESPO->setCorreioEletronico(str_replace('|','',$val['NMEMAIL']));
        }

    }

    private function _preparaRegistroDECPJ(){
        
        try{
            $params = array(
                "nrorg"            => $this->_nrOrg,
                "dtcompetencia"    => $this->_dtCompetencia,
                "nrestrresponsa"   => $this->_nrEstrResponsa,
                "idnaturezadecl"   => $this->_idNaturezaDecl,
                "idsocioostensi"   => $this->_idSocioOstensi,
                "iddepositjudic"   => $this->_idDepositJudic,
                "idinstitinvest"   => $this->_idInstitInvest,
                "idpagrendiexte"   => $this->_idPagRendiExte,
                "idpagplanosaud"   => $this->_idPagPlanoSaud,
                "idpagrelaccopa"   => $this->_idPagRelacCopa,
                "idpagrelacolimp"  => $this->_idPagRelacOlimp,
                "idpagsituaespe"   => $this->_idPagSituaEspe,
                "dteventoespeci"   => $this->_dtEventoEspeci,
            );
            $sqlResult = $this->queryDirf->get_decpj($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query, campo DECPJ, item 3.4.');
        }

        $registro = '';
        
        foreach($sqlResult as $val){
            if($val['CDCNPJ']          == '|')   //'IFs' para verificação de campos não preenchidos
                //MsgCalculo::getInstance()->adicionaMensagem( "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem CNPJ.");
                $this->msgErro .= "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem CNPJ.".Dirf::ql_msg;
            if($val['NOMEEMPRESARIAL'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem razão social.");
                $this->msgErro .= "Estrutura do responsável pela DIRF (nº ".$this->nrEstrResponsa.") sem razão social.".Dirf::ql_msg;
            if($val['IDNATUREZADECLA'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Não há natureza do declarante (formulário, estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Não há natureza do declarante (formulário, estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['CDCPFRESP']       == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Responsável perante o CNPJ não cadastrado ou sem CPF (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Responsável perante o CNPJ não cadastrado ou sem CPF (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDSOCIOSTENS']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de sócio ostensivo (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta indicador de sócio ostensivo (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDDEPOJUDIC']     == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de depósito de crédito decorrente de decisão judicial (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta indicador de depósito de crédito decorrente de decisão judicial (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDINSTINVEST']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de declarante de instituição administradora (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta indicador de declarante de instituição administradora (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGRENDEXT']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de rendimentos pagos a residentes no exterior (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta indicador de rendimentos pagos a residentes no exterior (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGPLANSAUD']   == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de plano privado de assistência à saúde (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta indicador de plano privado de assistência à saúde (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if ($this->_anoReferencia == 2016 || $this->_anoReferencia == 2018 || $this->_anoReferencia == 2019 || $this->_anoReferencia == 2020 || $this->_anoReferencia == 2021) {
                if($val['IDPAGRELCOPA']    == '|')
                    //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de pagamentos relacionados à Copa das Confedêrações e do Mundo (estrutura nº ".$this->nrEstrResponsa.").");
                    $this->msgErro .= "Falta indicador de pagamentos relacionados à Copa das Confedêrações e do Mundo (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            } else {
                unset($val['IDPAGRELCOPA']);
            }        
            if ($this->_anoReferencia == 2016 || $this->_anoReferencia == 2017 || $this->_anoReferencia == 2018 || $this->_anoReferencia == 2019 || $this->_anoReferencia == 2020 || $this->_anoReferencia == 2021) {
                if ($val['IDPAGRELOLIMP'] == '|')
                    //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de pagamentos relacionados aos Jogos Olímpicos e Paraolímpicos (estrutura nº " . $this->nrEstrResponsa . ")." . Dirf::ql_msg);
                    $this->msgErro .= "Falta indicador de pagamentos relacionados aos Jogos Olímpicos e Paraolímpicos (estrutura nº " . $this->nrEstrResponsa . ")." . Dirf::ql_msg;
            } else {
                unset($val['IDPAGRELOLIMP']);
            }
            if($val['IDPAGSITESPE']    == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta indicador de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta indicador de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            if($val['IDPAGSITESPE'] == '|S' && $val['DTEVENTDECPJ'] == '|')
                //MsgCalculo::getInstance()->adicionaMensagem( "Falta data de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").");
                $this->msgErro .= "Falta data de situação especial de declaração (estrutura nº ".$this->nrEstrResponsa.").".Dirf::ql_msg;
            foreach ($val as $field=>$res) {
                if($field != 'NRREGIMEAPUR')
                $registro .= $res;
            }
            /* Tipo de Regime de Apuração (Competência ou Caixa) */
            $this->_nrRegimeApur = $val['NRREGIMEAPUR'];

            $this->_registroDECPJ->setCnpj(str_replace('|','',$val['CDCNPJ']));
            $this->_registroDECPJ->setNmEmpresarial(str_replace('|','',$val['NOMEEMPRESARIAL']));
            $this->_registroDECPJ->setIdNaturezaDecl(str_replace('|','',$val['IDNATUREZADECLA']));
            $this->_registroDECPJ->setCpfResponsavel(str_replace('|','',$val['CDCPFRESP']));
            $this->_registroDECPJ->setIdSocioOstensi(str_replace('|','',$val['IDSOCIOSTENS']));
            $this->_registroDECPJ->setIdDepositJudic(str_replace('|','',$val['IDDEPOJUDIC']));
            $this->_registroDECPJ->setIdInstitInvest(str_replace('|','',$val['IDINSTINVEST']));
            $this->_registroDECPJ->setIdPagRendiExte(str_replace('|','',$val['IDPAGRENDEXT']));
            $this->_registroDECPJ->setIdPagPlanoSaud(str_replace('|','',$val['IDPAGPLANSAUD']));
            $this->_registroDECPJ->setIdPagEntidades(str_replace('|','',$val['IDPAGRELCOPA']));
            $this->_registroDECPJ->setIdFundaDirPriv(str_replace('|','',$val['IDPAGRELOLIMP']));
            $this->_registroDECPJ->setIdPagSituaEspe(str_replace('|','',$val['IDPAGSITESPE']));
            $this->_registroDECPJ->setDtEventoEspeci(str_replace('|','',$val['DTEVENTDECPJ']));
        }

    }
    
    private function _fimDirf(){
        return 'FIMDIRF|';
    }

}