<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 18/08/2017
 * Time: 11:09
 */

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\ESocialService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\MsgProcesso;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Util\HCM;

class ESocialController extends Crud {

    // Parametros de Rotas
    const IDTRANSMISSAO = 'idTransmissao';

    // Operações do eSocial
    const IDTRANSENVIO = 'ENVIAR'; //Enviar Evento
    const IDTRANSCONSU = 'CONSULTAR'; //Consultar Status do evento
    const IDTRANSREENV = 'REENVIAR'; //Reenviar Evento
    const IDTRANSEXCLU = 'EXCLUSAO'; //Excluir Evento
    const IDTRANSRETIF = 'RETIFICACAO'; //Retificar Evento


    protected $dataSourceName = '';

    protected $dts_Evento = 'getEventosESocial';
    //protected $dts_EnvioCadastros_get = 'eso_enviocadastros_get';
    protected $dts_EnvioCadastros_get = 'getEnviosCadastros';
    protected $dts_EnvioCadastrosIni_get = 'eso_enviocadastrosini_get';
    protected $dts_EnvioCadastros_set = 'eso_enviocadastros_set';
    
    protected $getEsonaoperiodico = 'getEsonaoperiodico';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRESTRUTURAM', 'P_CDOPERADOR');

    protected $qtdMaxEventosLote;
    
    // Array que armazena as mensagens geradas na rotina 'processEnvio'. É organizada da seguinte forma: 'nomeMensagem' => 'valoresAssociados'
    protected $mensagensProcessEnvio = array();

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->eSocialService = $eSocialService;

        $this->qtdMaxEventosLote = 50;
        parent::__construct($dataSourceManager);
    }
    
    public function deleteEventos(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $retorno = false;
            foreach ($request->getDataSet()->getRows() as &$row) {
                //$this->eSocialService->beforeDeleteEnvioCadastros($row);
                $this->eSocialService->deleteEventos($row);
                $retorno = $this->eSocialService->inativaEvento($row['NRORG'], $row['NRESTRUTURAM'], $row['NRENVIO']);
            }
            if($retorno){
                $response->addNotification(new DTO\Response\Notification('deleteEnvioCadastros', DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVersoesESocial (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $idativo = null;
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                if ($condition['columnName'] == 'IDATIVO') {
                    $idativo = $this->processConditionValue($condition);
                }
            }
            $dataSet = $this->eSocialService->getVersoesESocial($idativo);
            $dataSet = new DataSet($request->getRoutePath(), $dataSet);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getEstruturasESocial (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrtipoestrutlegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
            
            $filters = $request->getFilterCriteria()->getConditions();
            $idGrupo = \HCM\Util\Util::getValueInFilterDataSourceByName($filters, array('P_IDGRUPO'), true);
            \HCM\Util\Util::getValueInFilterDataSourceByName($filters, array('P_NRORG','P_NRORG_PADRAO','P_DTMESCOMPETENC'), true);
            $conditions = \HCM\Util\Util::createQueryFiltered($filters);
            
            $dataSet = $this->eSocialService->getEstruturasESocial($nrorg, $dtmescompetenc, $nrtipoestrutlegal, $idGrupo, $conditions);
            $dataSet = new DataSet($request->getRoutePath(), $dataSet);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getEventosESocial (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dts_Evento);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getEnviosCadastros (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $conditions = array();
            foreach ($filterData->getConditions() as $condition) {
                $columnName = $condition['columnName'];
                if (in_array($columnName, $this->parameterBagColumns)) {
                    $this->parameterBag->set($columnName, $this->processConditionValue($condition));
                } else {
                    if($columnName == 'NREVENTO_LIST'){
                        $condition['columnName'] = 'NREVENTO';
                    }
                    $conditions[] = $condition;
                }
            }
            $newFilterData = new FilterCriteria(
                $this->dts_EnvioCadastros_get,
                $conditions,
                null,
                7000
            );
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getEsonaoperiodico(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $conditions = array();
            foreach ($filterData->getConditions() as $condition) {
                $columnName = $condition['columnName'];
                if (in_array($columnName, $this->parameterBagColumns)) {
                    $this->parameterBag->set($columnName, $this->processConditionValue($condition));
                } else {
                    if($columnName == 'NREVENTO_LIST'){
                        $condition['columnName'] = 'NREVENTO';
                    }
                    $conditions[] = $condition;
                }
            }
            $newFilterData = new FilterCriteria(
                $this->dts_EnvioCadastros_get,
                $conditions,
                null,
                7000
            );
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getXmlEsocial (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $xmlFile = null;
            $idEvento = null;
            $dsDirXml = null;
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                if ($condition['columnName'] == 'P_XMLFILE') {
                    $xmlFile = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_IDEVENTO') {
                    $idEvento = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_DSDIRXML') {
                    $dsDirXml = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_NRTPAMBIENTE') {
                    $nrTpAmbiente = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_IDGRUPO') {
                    $idGrupo = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_CDCNPJESTRUT') {
                    $cdCnpjEstrut = $this->processConditionValue($condition);
                }
            }
            // Alteração temporária para ler os arquivos no servidor AWS
            $dsDirXmlFilesPath = 
                ($nrTpAmbiente == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                ($idGrupo == 'P' ? 'Periodicos' : ($idGrupo == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                $cdCnpjEstrut . "/" .
                ($xmlFile == 'envio' ? 'Envio' : ($xmlFile == 'retorno' ? 'RetornoEnvio' : ($xmlFile == 'consulta' ? 'RetornoProcessamento' : ($xmlFile == 'total' ? 'Totalizador' : '')))). "/" .
                $dsDirXml;
            $xml = $this->eSocialService->getXmlFiles($nrorg, $xmlFile, $idEvento, $dsDirXml, $dsDirXmlFilesPath);
            //$xml = $this->eSocialService->getXmlOnFilesPath($nrorg, $xmlFile, $idEvento, $dsDirXml);
            
            // -------
            $xmlDataSet = $this->eSocialService->setXmlFileToDataSet($xml);
            $dataSet = new DataSet($request->getRoutePath(), $xmlDataSet);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($dsDirXml, $e->getCode()));
        }
    }

    public function setEnvioCadastros(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $this->esvaziaMensagensProcessEnvio();
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->dataSourceName = $this->dts_EnvioCadastros_set;
                
                if ($row['__is_new']) {
                    $arrayEvento = array();
                    foreach($row['NREVENTO_LIST'] as $index => $nrevento){
                        $arrayEvento[$nrevento] = array('NMEVENTO' => $row['NMEVENTO'][$index],
                                                        'DSEVENTO' => $row['DSEVENTO'][$index],
                                                        'IDGRUPO'  => $row['IDGRUPO'][$index]);
                    }
                    $backRowChave = null;
                    $backRowTabela = null;
                    
                    foreach($arrayEvento as $nrevento => $arraydescricao){
                        $row['NMEVENTO'] = $arraydescricao['NMEVENTO'];
                        $row['NREVENTO'] = $nrevento;
                        $row['DSEVENTO'] = $arraydescricao['DSEVENTO'];
                        $row['IDGRUPO'] = $arraydescricao['IDGRUPO'];
                        $tabelas = null;$evento = null;$chaves = null;;$nomeChave = null;
                        switch($row['NMEVENTO']){
                            case 'S-1010':
                                $tabelas = $row['NRTIPOMOVIMENT'];
                                $chaves = $row['NREVENTOM'];
                                $nomeChave = 'NREVENTOM';
                                break;
                            case 'S-1030':
                            case 'S-1040':
                                $chaves = $row['NROCUPACAOM'];
                                $nomeChave = 'NROCUPACAOM';
                                break;
                            case 'S-1050':
                                $chaves = $row['NRHORDIAM'];
                                $nomeChave = 'NRHORDIAM';
                                break;
                            case 'S-1060':
                                $chaves = $row['NRESTRUTGEREN'];
                                $nomeChave = 'NRESTRUTGEREN';
                                break;
                            case 'S-1020':
                                $chaves = $row['NRESTRUTTOMAD'];
                                $nomeChave = 'NRESTRUTTOMAD';
                                break;
                            case 'S-2190':case 'S-2200':case 'S-2205':case 'S-2206':case 'S-2230':case 'S-2250':case 'S-2260':case 'S-2298':case 'S-2299': case 'S-2300': case 'S-2306':case 'S-2399':
                                $chaves = $row['NRVINCULOM'];
                                $nomeChave = 'NRVINCULOM';
                                break;
                            case 'S-1200':
                                $tabelas = array_key_exists('NRTIPOMOVIMENT', $row) ? $row['NRTIPOMOVIMENT'] : null;
                                $chaves = array_key_exists('NRPESSOA', $row) ? $row['NRPESSOA']: null;
                                $nomeChave = 'NRPESSOA';
                                break;
                            case 'S-1210':
                                $chaves = array_key_exists('NRPESSOA', $row) ? $row['NRPESSOA']: null;
                                $nomeChave = 'NRPESSOA';
                                break;
                            case 'S-1250':
                                $chaves = $row['NRESTRUTURAM'];
                                $nomeChave = 'NRESTRUTURAM';
                                break;
                        }
                        if(!empty($backRowChave)){
                            $chaves = $backRowChave;
                        }else{
                            $backRowChave = $chaves;
                        }
                        if(!empty($backRowTabela)){
                            $tabelas = $backRowTabela;
                        }else{
                            $backRowTabela = $tabelas;
                        }
                        $row['NRVINCULOM'] = $row['NRVINCULOM'] ? $row['NRVINCULOM'] : null;
                        if(!empty($tabelas)){
                            foreach ($tabelas as $tabela) {
                                $row['NRTIPOMOVIMENT'] = $tabela;
                                if($chaves){
                                    $chaves = is_array($chaves) ? $chaves : [$chaves];
                                    foreach ($chaves as $chave) {
                                        $this->processEnvio($chave, $nomeChave, $row, $request, $response);
                                    }
                                }else{
                                    $this->processEnvio(null, $nomeChave, $row, $request, $response);
                                }
                            }
                        }else if(!empty($chaves)){
                            $row['NRTIPOMOVIMENT'] = null;
                            $chaves = is_array($chaves) ? $chaves : [$chaves];
                            $rescisao = array_key_exists('DTRESCISAOVINC', $row) ? $row['DTRESCISAOVINC'] : null;
                            foreach ($chaves as $index => $chave) {
                                $row['DTRESCISAOVINC'] = is_array($rescisao) ? $rescisao[$index] : null;
                                $this->processEnvio($chave, $nomeChave, $row, $request, $response);                                
                            }
                        }else{
                            $row['NRTIPOMOVIMENT'] = null;
                            $this->processEnvio(null, $nomeChave, $row, $request, $response);
                        }
                    }
                }
                else {
                    if (!array_key_exists('NRVINCULOM', $row)) {
                        $row['NRVINCULOM'] = $row['IDGRUPO'] == 'N' ? $row['DSCDCHAVE'] : null;
                    }
                    //Tratativa para quando foi alterado o protocolo de envio e o ID do histórico está vazio, para não deixar o ID vazio
                    if (is_null($row['IDEVENTO'])) {
                        if (!is_null($row['DSDIRXMLENV'])) {
                            $row['IDEVENTO'] = substr(str_replace('envio_','',$row['DSDIRXMLENV']),0,36);
                        } else {
                            $envio = $this->eSocialService->getEnvioCadastro($row['NRORG'],$row['NRENVIO']);
                            $row['IDEVENTO'] = $envio->getIdevento();
                        }
                    }
                    
                    parent::save($request, $response);
                }
            }
            
            if($this->existemMensagensProcessEnvio()){
                $response->addMessage(new DTO\Response\Message($this->getMensagensProcessEnvio(), DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteEnvioCadastros(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $retorno = false;
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->eSocialService->beforeDeleteEnvioCadastros($row);
                $this->eSocialService->deleteDadosEnvio($row);
                $retorno = $this->eSocialService->inativaEvento($row['NRORG'], $row['NRESTRUTURAM'], $row['NRENVIO']);
            }
            if($retorno){
                $response->addNotification(new DTO\Response\Notification('deleteEnvioCadastros', DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setTransmissaoESocial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            $oldmask = umask(0);
            
            $msgRetorno = '';
            $msgValidacao = array();
            $idTransmissao = $request->getParameter(self::IDTRANSMISSAO);
            $idGrupo = '';
            $dsVersao = '';
            $mensagensTransmissao = "";
            
            $this->eSocialService->entityManager->beginTransaction();
            
            if (count($request->getDataSet()->getRows()) < 1) {
                throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
            }
            
            $paramESocial = $this->eSocialService->getParamESocial($request->getDataSet()->getRows()[0]['NRESTRUTURAM']);
            $this->eSocialService->setDadosTransmissao($request->getDataSet()->getRows()[0]['NRESTRUTURAM']);

            // variavel responsavel por decidir se será realizada a consulta ao final do processo de envio dos respectivos eventos
            // todos os eventos deverão possuir a posição REALIZACONSULTA marcada com true para que seja realizado a consulta
            $realizaConsulta = array_reduce($request->getDataSet()->getRows(), function ($acumulador, $evento) {
                return $acumulador && $evento['REALIZACONSULTA'];
            }, true);
            
            if ($idTransmissao == self::IDTRANSENVIO || $idTransmissao == self::IDTRANSREENV) {
                $rows =  $request->getDataSet()->getRows();
                $eventosLote = array();
                $eventosNaoPeriodicosPendentes = array();
                // verifica caso seja um evento 1200 ou 1210 se não existem periódicos pendentes de envio
                $realizaVerificacao = $rows[0]['REALIZAVERIFICACAO'];
                if($realizaVerificacao){
                    foreach ($rows as &$row) {
                        $eventosPendentes = $this->eSocialService->getPendenciasEnvio($row['NMEVENTO'], $row['NRCHAVE'], $row['NRORG']);
                        
                        // Valida possíveis pendências para o envio
                        if(!empty($eventosPendentes)){
                            $eventosNaoPeriodicosPendentes = array_merge($eventosNaoPeriodicosPendentes, $eventosPendentes);
                            continue;
                        }
                    }
                }
                
                if(($realizaVerificacao && count($eventosNaoPeriodicosPendentes) == 0) || !$realizaVerificacao ){
                    foreach ($rows as &$row) {
                        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
                        
                        if($idTransmissao == self::IDTRANSREENV){
                            $this->eSocialService->deleteDadosEnvio($row);
                        }
                        
                        /*Reenvio de evento de exclusão*/
                        $envioOrig = $this->eSocialService->getEnvioCadastroOrigemExcl($row['NRORG'], $row['NRENVIO']);
                        $row['NRENVIOORIG'] = is_object($envioOrig) ? $envioOrig->getNrenvio() : null;
                        $row['NRTABELAORIG'] = is_object($envioOrig) ? $envioOrig->getNrtabela() : null;
                        $row['DTMESCOMPORIG'] = is_object($envioOrig) ? $envioOrig->getDtmescompetenc() : null;
                        $row['NRRECIBO2ORIG'] = is_object($envioOrig) ? $envioOrig->getNrrecibo2() : null;
                        $row['DTRESPOSTAORIG'] = is_object($envioOrig) ? $envioOrig->getDtresposta()->format('d/m/Y') : null;
                        $eventoOrig = is_object($envioOrig) ? $this->eSocialService->getEvento($envioOrig->getNrevento()) : null;
                        $row['NMEVENTOORIG'] = is_object($eventoOrig) ? $eventoOrig->getNmevento() : null;
                        
                        $evento = $this->eSocialService->getEventoPorNomeVersao($row['NMEVENTO'], $row['DSVERSAO']);
                        $idGrupo = is_object($evento) ? $evento->getIdgrupo() : $row['IDGRUPO'];
                        $dsVersao = str_replace('.','-',$row['DSVERSAO']);
                        $row['IDGRUPO'] = $idGrupo;
                        
                        
                        $beforeEnvio = $this->eSocialService->beforeEnvioEvento(
                            $row['NRORG'], $row['NRESTRUTURAM'], $row['NRTPAMBIENTE'], $paramESocial, $evento->getNrevento(), $row['NRCHAVE'], $row['NRTABELA'], $row['IDOPERACAO'], $row['DTMESCOMPETENC']
                        );
                        $xmlEvento = $this->eSocialService->getXmlEvento($dsVersao, $row);
                        if ($xmlEvento) {
                            $eventosLote[] = $xmlEvento;
                        }
                        
                        
                        foreach ($msgProcesso->getMessages() as $msg) {
                            $msgValidacao[] = array(
                                'NMEVENTO' => $row['NMEVENTO'],
                                'NRCHAVE' => $row['NRCHAVE'],
                                'MSG' => $msg['MSG'],
                                'TYPE' => $msg['TYPE'],
                            );
                        }
                        $msgProcesso->clear();
                        unset($msgProcesso);
                        
                    }
                }
                    
                
                if (empty($msgValidacao) && empty($eventosLote) && empty($eventosNaoPeriodicosPendentes)) {
                    // caso não existam nem eventos no lote nem pendências
                    throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
                } else if (empty($msgValidacao)) {
                    
                    // verifica se existem pendencias
                    // caso existam, os eventos são retornados
                    if (!empty($eventosNaoPeriodicosPendentes) && $realizaVerificacao) {
                        // adiciona um dataSet para que as pendencias sejam tratadas
                        // não interrompem o processo caso outros eventos sem pendências estejam sendo enviados simultânamente
                        
                        // exclui posicões iguais no array e orderna pelo número do vínculo (DSCDCHAVE)
                        $eventosNaoPeriodicosPendentes = \HCM\Util\Util::unique_multidim_array($eventosNaoPeriodicosPendentes);
                        usort($eventosNaoPeriodicosPendentes, function ($a, $b) {
                            return strcmp(strtoupper($a['DSCHAVE']), strtoupper($b['DSCHAVE'])) > 0;
                        });
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('pendenciasDeEnvioPeriodicos', $eventosNaoPeriodicosPendentes));
                        $msgRetorno .= 'pendenciasDeEnvioPeriodicos';
                    } else if (!empty($eventosLote)) {
                    // se existem eventos no lote, são enviados
                    // eventos com pendências não são enviados (não estão no array de lote)
                        $xmlLoteEventos = $this->eSocialService->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventosLote);
                        
                        $retorno = $this->eSocialService->enviarLote($eventosLote, $xmlLoteEventos);
                        
                        if ($realizaConsulta && $retorno['COD'] == '201') {
                            $this->eSocialService->entityManager->flush();
                            $this->eSocialService->entityManager->commit();
                            $this->eSocialService->entityManager->beginTransaction();
                    
                            $nrRecibo = $retorno['REC']; // Recibo do lote
                            $dsVersao = str_replace('.','-',$row['DSVERSAO']);
                            
                            $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                            $retorno = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                            
                            if ($retorno['COD'] == '101') {
                                // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                                sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                                $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                                $retorno = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                            }
                            
                        }
                        
                        $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
                    }
                    
                }
            } elseif ($idTransmissao == self::IDTRANSCONSU) {
                foreach ($request->getDataSet()->getRows() as &$row) {
                    //Verifica se é consulta do evento de exclusão do empregador
                    $idExclusaoEmpreg = false;
                    if (strpos($row['DSOBSERVACAO'],'RemoverEmpregadorDaBaseDeDadosDaProducaoRestrita') !== false) {
                        $idExclusaoEmpreg = true;
                    }
                    
                    $dsVersao = str_replace('.','-',$row['DSVERSAO']);
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($row['NRRECIBO'], $dsVersao);
                    $retorno = $this->eSocialService->consultarEvento($row['NRRECIBO'], $xmlConsulta, $idExclusaoEmpreg);
                    $msgRetorno = $retorno['COD'] . '-' . $retorno['MSG'];
                    
                    // Se for o evento S-1000 de exclusão do empregador do ambiente de produção restrita
                    if ($row['NMEVENTO'] == 'S-1000' && ($retorno['COD'] == '1012' or $retorno['COD'] == '255') && $row['NRTPAMBIENTE'] == ESocialService::TPAMB_PROD_REST_REAL) {
                        // Desativa todos eventos do empregador
                        $this->eSocialService->excluirEventos($row['NRESTRUTURAM'], $paramESocial['NRINSCRICAOPARC'], $paramESocial['DSVERSAO'], $row['NRTPAMBIENTE']);
                        $msgRetorno = 'Exclusão do empregador da base de dados da produção restrita realizada com sucesso [' . $retorno['COD'] . '-' . $msgRetorno . '].';
                    }
                }
            } elseif ($idTransmissao == self::IDTRANSEXCLU || $idTransmissao == self::IDTRANSRETIF) {
                $eventosLote = array();
                $idGrupo = null;
                $nrvincm = null;
                foreach ($request->getDataSet()->getRows() as &$row) {
                    $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRENVIO']);
                    
                    if ($row['IDGRUPO'] == 'I' || 
                        $row['IDGRUPO'] == 'T' || 
                        ($idTransmissao == self::IDTRANSRETIF && ($row['IDGRUPO'] == 'P' || $row['IDGRUPO'] == 'N'))) {
                        $evento = $this->eSocialService->getEventoPorNomeVersao($row['NMEVENTO'], $paramESocial['DSVERSAO']);
                    } else {
                        $evento = $this->eSocialService->getEventoPorNomeVersao('S-3000', $paramESocial['DSVERSAO']);
                    }
                    
                    $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
                    $idGrupo = $evento->getIdgrupo();
                    // Incluí o evento de exclusão/retificação referente ao envio que será excluído/retificado
                    $idOperacao = ($idTransmissao == self::IDTRANSRETIF ? 'R' : 'E');
                    
                    $envioOrig = $this->eSocialService->getEnvioCadastro($row['NRORG'],$row['NRENVIO']);
                    if (!is_object($envioOrig)) {
                        throw new \Exception('Envio não encontrado. Favor atualizar os dados da tela e tentar novamente.');
                    }
                    if ($envioOrig->getNrenvioexcl()) {
                        throw new \Exception('O envio já foi retificado ou excluído. Favor atualizar os dados da tela e tentar novamente.');
                    }

                    if(($row['NMEVENTO'] == 'S-1200' || $row['NMEVENTO'] == 'S-1210' || $row['NMEVENTO'] == 'S-1250') && $evento->getNmevento() == 'S-3000') {
                        $nrvincm = null;
                    } else {
                        $nrvincm = $idGrupo == 'N' && array_key_exists('DSCDCHAVE', $row) ? $row['DSCDCHAVE'] : null;
                    }
                    
                    $envioexcl = $this->eSocialService->novoEsoEnvioCadastro(
                        $evento->getNrevento(), $row['NRESTRUTURAM'], $row['NRCHAVE'], $row['NRTABELA'], $row['DTMESCOMPETENC'], $idOperacao, $paramESocial['NRTPAMBIENTE'], null, $nrvincm
                    );
                    
                    $arrEnvioExcl = $envioexcl->toArray();
                    $arrEnvioExcl['NMEVENTO'] = $evento->getNmevento();
                    $arrEnvioExcl['IDGRUPO'] = $evento->getIdgrupo();
                    $arrEnvioExcl['NRENVIOORIG'] = $row['NRENVIO'];
                    $arrEnvioExcl['NMEVENTOORIG'] = $row['NMEVENTO'];
                    $arrEnvioExcl['NRTABELAORIG'] = $row['NRTABELA'];
                    $arrEnvioExcl['DTMESCOMPORIG'] = \HCM\Util\DateUtil::getDataDeString($row['DTMESCOMPETENC'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true);
                    $arrEnvioExcl['NRRECIBO2ORIG'] = $row['NRRECIBO2'];
                    $arrEnvioExcl['DTRESPOSTAORIG'] = \HCM\Util\DateUtil::getDataDeString($row['DTRESPOSTA'], \HCM\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format('d/m/Y');
                    
                    $arrEnvioExcl['DTMESCOMPETENC'] = is_object($arrEnvioExcl['DTMESCOMPETENC']) ? $arrEnvioExcl['DTMESCOMPETENC']->format('d/m/Y H:i:s') : $arrEnvioExcl['DTMESCOMPETENC'];
                    $xmlEvento = $this->eSocialService->getXmlEvento($dsVersao, $arrEnvioExcl);
                    
                    if ($xmlEvento) {
                        $eventosLote[] = $xmlEvento;
                    }
                    
                    foreach ($msgProcesso->getMessages() as $msg) {
                        $msgValidacao[] = array(
                            'NMEVENTO' => $row['NMEVENTO'],
                            'NRCHAVE' => $row['NRCHAVE'],
                            'MSG' => $msg['MSG'],
                            'TYPE' => $msg['TYPE'],
                        );
                    }
                    $msgProcesso->clear();
                    unset($msgProcesso);
                    
                    // Atualiza o evento a ser excluído com o número do envio de exclusão
                    $envio = $this->eSocialService->setEventoExclusaoEventoOrigem($this->hcmParameters->getNrorgTrabalho(), $envioexcl->getNrenvio(), $row['NRENVIO'], $row['IDGRUPO'], $row['NREVENTO']);
                }
                
                if (empty($msgValidacao) && empty($eventosLote)) {
                    throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
                }

                $xmlLoteEventos = $this->eSocialService->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventosLote);

                $retorno = $this->eSocialService->enviarLote($eventosLote, $xmlLoteEventos);
                
                $codRetorno = $retorno['COD'];
                $nrRecibo = $retorno['REC'];
                $msgRetorno = ($idTransmissao == self::IDTRANSRETIF ? 'Envio da Retificação: ' : 'Envio da Exclusão: ') . $retorno['MSG'];
                
                // Caso o evento de exclusão/retificação seja enviado com sucesso realiza a consulta automática
                if ($codRetorno == '201') {
                    $this->eSocialService->entityManager->flush();
                    $this->eSocialService->entityManager->commit();
                    $this->eSocialService->entityManager->beginTransaction();
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                    $retornoConsulta = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                    $codRetornoConsulta = $retornoConsulta['COD'];
                    $msgRetornoConsulta = $retornoConsulta['MSG'];
                    $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    if ($codRetornoConsulta == '101') {
                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                        sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                        $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                        $retornoConsulta = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                        $codRetornoConsulta = $retornoConsulta['COD'];
                        $msgRetornoConsulta = $retornoConsulta['MSG'];
                        $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    }
                }
                $msgRetorno = $msgRetorno . ($idTransmissao == self::IDTRANSRETIF ? ' / Consulta da Retificação: ' : ' / Consulta da Exclusão: ') . $codRetornoConsulta . '-' . $msgRetornoConsulta . ($tmpEsperaConsulta ? '(Tempo de Espera:'.$tmpEsperaConsulta.')' : '');
            }
            
            if (count($msgValidacao) > 0) {
                $this->eSocialService->entityManager->flush();
                $this->eSocialService->entityManager->commit();
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('setTransmissaoESocial', $msgValidacao));
            } elseif ($msgRetorno) {
                $this->eSocialService->entityManager->flush();
                $this->eSocialService->entityManager->commit();
                $response->addMessage(new DTO\Response\Message($msgRetorno));
            } else {
                $this->eSocialService->entityManager->rollback();
                $response->addMessage(new DTO\Response\Message("Serviço '$idTransmissao' não disponível", DTO\Response\Message::TYPE_WARNING));
            }
            umask($oldmask);
        } catch(\Exception $e) {
            $this->eSocialService->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function gerarCargaInicial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $oldmask = umask(0);

            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->eSocialService->entityManager->beginTransaction();

                $nrEstruturam = $row['NRESTRUTURAM'];
                $idMatriz = $row['IDMATRIZ'];
                $idGrupo = $row['IDGRUPO'];

                $paramESocial = $this->eSocialService->getParamESocial($nrEstruturam);

                // Verifica se a estrutura tem parâmetro do eSocial
                if (empty($paramESocial)) {
                    throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR);
                }

                $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
                $nrtpambiente = $paramESocial['NRTPAMBIENTE'];

                // Verifica se a estrutura já fez o envio da carga inicial
                if ($nrtpambiente == ESocialService::TPAMB_PROD && $idGrupo == 'T' && !empty($paramESocial['DTTRANSMISSAO'])) {
                    throw new \Exception(Messages::HCM09000_ESO_CARGAINI_EXISTS_ERROR);
                }

                // Busca os eventos que irão compor a carga inicial
                $eventos = array();
                if ($idGrupo == 'T') {
                    $conditions = array(
                        array( 'columnName' => 'DSVERSAO', 'operator' => '=', 'value' => $paramESocial['DSVERSAO'] ),
                        array( 'columnName' => 'IDGRUPO', 'operator' => 'IN', 'value' => array('I','T') ),
                        array( 'columnName' => 'IDATIVO', 'operator' => '=', 'value' => 'S' ),
                        array( 'columnName' => 'NMEVENTO', 'operator' => '<>', 'value' => 'S-1060' ),
                    );
                }
                else if ($idGrupo == 'N') {
                    $conditions = array(
                        array( 'columnName' => 'DSVERSAO', 'operator' => '=', 'value' => $paramESocial['DSVERSAO'] ),
                        array( 'columnName' => 'NMEVENTO', 'operator' => 'IN', 'value' => array('S-2200','S-2300') ),
                        array( 'columnName' => 'IDATIVO', 'operator' => '=', 'value' => 'S' ),
                    );
                }
                else {
                    $conditions = array(
                        array( 'columnName' => 'DSVERSAO', 'operator' => '=', 'value' => $paramESocial['DSVERSAO'] ),
                        array( 'columnName' => 'NMEVENTO', 'operator' => '=', 'value' => '' ),
                        array( 'columnName' => 'IDATIVO', 'operator' => '=', 'value' => 'S' ),
                    );
                }
                $newFilterData = new FilterCriteria($this->dts_Evento, $conditions, 1, 50);
                $dataSet = $this->dataSourceManager->findBy($newFilterData);
                $eventos = array_merge($eventos, $dataSet->getRows());
                // Eventos S-2200 e S-2300
                /*$conditions = array(
                    array( 'columnName' => 'DSVERSAO', 'operator' => '=', 'value' => $paramESocial['DSVERSAO'] ),
                    array( 'columnName' => 'NMEVENTO', 'operator' => 'IN', 'value' => array('S-2200','S-2300') ),
                    array( 'columnName' => 'IDATIVO', 'operator' => '=', 'value' => 'S' ),
                );
                $newFilterData = new FilterCriteria($this->dts_Evento, $conditions, 1, 50);
                $dataSet = $this->dataSourceManager->findBy($newFilterData);
                $eventos = array_merge($eventos, $dataSet->getRows());*/

                foreach($eventos as $evento) {
                    // Busca os itens que serão enviados por evento
                    try {
                        $arrItens = $this->eSocialService->getItensParaEnvioEvento(
                            $evento['NMEVENTO'],
                            $evento['IDGRUPO'],
                            array('NRESTRUTURAM' => $nrEstruturam, 'NRTPAMBIENTE' => $nrtpambiente, 'IDMATRIZ' => $idMatriz)
                        );
                    } catch (\Exception $e) {
                        if ($e->getMessage() == 'HCM09000_ESO_CALLED_EVENTO_ERROR' || $e->getMessage() == 'HCM09000_ESO_EVENTO_MATRIZ_ERROR') {
                            continue;
                        } else {
                            throw $e;
                        }
                    }
                    foreach($arrItens as $item) {
                        //TODO Verificar se já existe envio cadastrado para o item
                        try {
                            $row['NRVINCULOM'] = array_key_exists('NRVINCULOM', $row) ? $row['NRVINCULOM'] : null; 
                            $this->eSocialService->beforeInsertEnvioCadastros($nrEstruturam, $nrtpambiente, $evento['NREVENTO'], $item['NRCHAVE'], $item['NRTABELA'], $item['IDOPERACAO'], $item['DTMESCOMPETENC']);
                        } catch (\Exception $e) {
                            if ($e->getMessage() == Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR) {
                                continue;
                            } else {
                                throw $e;
                            }
                        }

                        $envio = $this->eSocialService->novoEsoEnvioCadastro(
                            $evento['NREVENTO'], $nrEstruturam, $item['NRCHAVE'], $item['NRTABELA'], $item['DTMESCOMPETENC'], $item['IDOPERACAO'], $nrtpambiente, $row['NRVINCULOM']
                        );
                    }
                }

                if ($nrtpambiente == ESocialService::TPAMB_PROD) {
                    $this->eSocialService->atualizaDataCargaInicial($paramESocial['NRPARAMETRO']);
                }

                $this->eSocialService->entityManager->flush();
                $this->eSocialService->entityManager->commit();
            }

            umask($oldmask);

            $response->addNotification(new DTO\Response\Notification('gerarCargaInicial', DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $this->eSocialService->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function excluirDadosEmpregador(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);

            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->eSocialService->entityManager->beginTransaction();

                $nrEstruturam = $row['NRESTRUTURAM'];
                $cnpjEstrutura = $row['CDCNPJESTRUT'];

                $paramESocial = $this->eSocialService->getParamESocial($nrEstruturam);

                // Verifica se a estrutura tem parâmetro do eSocial
                if (empty($paramESocial)) {
                    throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR);
                }
                
                $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
                $nrtpambiente = $paramESocial['NRTPAMBIENTE'];

                // Verifica se o ambiente é o de produção restrita
                if ($nrtpambiente != ESocialService::TPAMB_PROD_REST_REAL && $nrtpambiente != ESocialService::TPAMB_PROD_REST_FICT) {
                    throw new \Exception(Messages::HCM09000_ESO_PRODREST_OPER_ERROR);
                }
                
                $this->eSocialService->setDadosTransmissao($nrEstruturam);
                
                // Incluí o evento de exclusão do empregador
                $evento = $this->eSocialService->getEventoPorNomeVersao('S-1000', $row['DSVERSAO']);
                $envio = $this->eSocialService->novoEsoEnvioCadastro(
                    $evento->getNrevento(), $nrEstruturam, $nrEstruturam, '', $row['DTMESCOMPETENC'], 'I', $nrtpambiente
                );

                if (is_object($envio)) {
                    // Monta o xml do evento de exclusão do empregador
                    $xmlEvento = $this->eSocialService->getXmlEventoExclusaoEmpregador($dsVersao,
                        array(
                            'NRENVIO' => $envio->getNrenvio(),
                            'NRESTRUTURAM' => $nrEstruturam,
                            'NRCHAVE' => $nrEstruturam,
                            'NMEVENTO' => 'S-1000',
                            'IDGRUPO' => 'I',
                            'IDEVENTO' => null,
                            'IDOPERACAO' => 'I',
                            'NRTPAMBIENTE' => $nrtpambiente,
                            'DTMESCOMPETENC' => $row['DTMESCOMPETENC']
                        )
                    );
                
                    if (!$xmlEvento) {
                        throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
                    }
                } else {
                    throw new \Exception("Erro ao gerar envio de exclusão do empregador");
                }

                // Monta o xml do lote para envio do evento de exclusão do empregador
                $xmlLoteEventos = $this->eSocialService->getXmlLote('I', $paramESocial, $dsVersao, array($xmlEvento));

                // Envia o lote com o evento
                $retorno = $this->eSocialService->enviarLote(array(), $xmlLoteEventos);
                $codRetorno = $retorno['COD'];
                $nrRecibo = $retorno['REC'];
                $msgRetorno = $retorno['MSG'];
                
                // Verifica o retorno do envio - 201:Enviado com sucesso
                if ($codRetorno == '201') {
                    // Atualiza o status do envio
                    $envio->setIdstatus('M');
                    $envio->setIdevento($xmlEvento['IDEVENTO']);
                    $envio->setNrrecibo($nrRecibo);
                    $envio->setDtenvio(\HCM\Util\DateUtil::getDataAtual());
                    $envio->setDtresposta(\HCM\Util\DateUtil::getDataAtual());
                    //$envio->setDsdirxmlenv($xmlPath);
                    $envio->setDsobservacao('RemoverEmpregadorDaBaseDeDadosDaProducaoRestrita');
                    $envio->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
                    $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->eSocialService->entityManager->persist($envio);
                    
                    // Realiza consulta automática do envio realizado
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                    $retorno = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta, true);
                    $codRetorno = (string)$retorno['COD'];
                    $msgRetorno = $retorno['MSG'];
                    $tmpEspera = $retorno['TMP'];
                    $nrRecibo2 = $retorno['REC'];
                    if ($codRetorno == '101') {
                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                        sleep($tmpEspera ? (int)$tmpEspera : 60);
                        $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                        $retorno = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta, true);
                        $codRetorno = (string)$retorno['COD'];
                        $msgRetorno = $retorno['MSG'];
                        $nrRecibo2 = $retorno['REC'];
                    }
                    
                    if ($codRetorno == '101') {
                        // Atualiza o status do envio
                        $envio->setIdstatus('M');
                        $envio->setCderro($codRetorno);
                        $envio->setDsobservacao($envio->getDsobservacao().'. '.$msgRetorno);
                        $envio->setDtprocessamento(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->eSocialService->entityManager->persist($envio);
                        
                        $this->eSocialService->entityManager->flush();
                        $this->eSocialService->entityManager->commit();
                        
                        $response->addMessage(new DTO\Response\Message($codRetorno.'-'.$msgRetorno, DTO\Response\Message::TYPE_WARNING));
                        return;
                    }
                    // O Código 1012 é o esperado confirmando a exclusão dos dados
                    elseif ($codRetorno == '1012') {
                        // Atualiza o status do envio
                        $envio->setIdstatus('C');
                        $envio->setCderro($codRetorno);
                        $envio->setDtprocessamento(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setNrrecibo2($nrRecibo2);
                        $envio->setIdativo('N');
                        $envio->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->eSocialService->entityManager->persist($envio);
                        
                        // Desativa todos eventos do empregador
                        $this->eSocialService->excluirEventos($nrEstruturam, $cnpjEstrutura, $paramESocial['DSVERSAO'], $nrtpambiente);
                        
                        $this->eSocialService->entityManager->flush();
                        $this->eSocialService->entityManager->commit();
                        
                        $response->addMessage(new DTO\Response\Message($msgRetorno));
                    // O Código 255 é o esperado quando os dados já foram excluídos
                    } elseif ($codRetorno == '255') {
                        // Atualiza o status do envio
                        $envio->setIdstatus('C');
                        $envio->setCderro($codRetorno);
                        $envio->setDtprocessamento(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setIdativo('N');
                        $envio->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->eSocialService->entityManager->persist($envio);
                        
                        // Desativa todos eventos do empregador
                        $this->eSocialService->excluirEventos($nrEstruturam, $cnpjEstrutura, $paramESocial['DSVERSAO'], $nrtpambiente);
                        
                        $this->eSocialService->entityManager->flush();
                        $this->eSocialService->entityManager->commit();
                        
                        //$response->addMessage(new DTO\Response\Message($msgRetorno));
                        $response->addMessage(new DTO\Response\Message('Empregador já havia sido removido da base de produção restrita.'));
                    } else {
                        // Atualiza o status do envio
                        $envio->setIdstatus('R');
                        $envio->setCderro($codRetorno);
                        $envio->setDsobservacao($envio->getDsobservacao().'. '.$msgRetorno);
                        $envio->setDtprocessamento(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setDtultatu(\HCM\Util\DateUtil::getDataAtual());
                        $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $this->eSocialService->entityManager->persist($envio);
                        
                        $this->eSocialService->entityManager->flush();
                        $this->eSocialService->entityManager->commit();
                        
                        //throw new \Exception($codRetorno.'-'.$msgRetorno);
                        $response->addMessage(new DTO\Response\Message($codRetorno.'-'.$msgRetorno, DTO\Response\Message::TYPE_ERROR));
                        return;
                    }
                } else {
                    throw new \Exception($codRetorno.'-'.$msgRetorno);
                }
            }
            
            $response->addNotification(new DTO\Response\Notification('excluirDadosEmpregador', DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $this->eSocialService->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function envioAutomaticoEventos(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            /*LOG -*/
            $start = microtime(true);
            $this->eSocialService->saveLogESocial('=== envioAutomaticoEventos starts ===');
            /*- LOG*/
            
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $oldmask = umask(0);

            $filter = current($request->getDataSet()->getRows());

            $nrEstruturam = $filter['NRESTRUTURAM'];
            $nrEvento = $filter['NREVENTO'];
            $nmEvento = $filter['NMEVENTO'];
            $idGrupo = $filter['IDGRUPO'];
            $idStatus = $filter['IDSTATUS'];
            $realizaVerificacao = $filter['REALIZAVERIFICACAO'];
            // $validacaoPendencias = $filter['VALIDACAOPENDENCIAS'];
            $dtmescompetenc = $this->hcmParameters->getDtcompetencia();
            
            /*LOG -*/
            $this->eSocialService->saveLogESocial('envioAutomaticoEventos Estrutura:' . $nrEstruturam . ' Evento:' . $nmEvento);
            /*- LOG*/

//            $this->eSocialService->entityManager->beginTransaction();

            $paramESocial = $this->eSocialService->getParamESocial($nrEstruturam);

            // Verifica se a estrutura tem parâmetro do eSocial
            if (empty($paramESocial)) {
                throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR);
            }

            $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
            $nrtpambiente = $paramESocial['NRTPAMBIENTE'];
            $nrtabela = array_key_exists('NRTIPOMOVIMENT', $filter) ? $filter['NRTIPOMOVIMENT'] : null;
            $nrtabela = !empty($nrtabela) ? $nrtabela : null;
            $nmEvento = $filter['NMEVENTO'];

            /*LOG -*/
            $lap = microtime(true);
            /*- LOG*/
            // Busca os itens pendentes que serão enviados por evento
            $arrEnvios = $this->eSocialService->getEnviosParaEvento($nrEstruturam, $nrtpambiente, $nrEvento, $idStatus, $dtmescompetenc, $idGrupo, $nmEvento, $nrtabela);
            if (empty($arrEnvios)) {
                throw new \Exception(Messages::HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR);
            }
            /*LOG -*/
            $this->eSocialService->saveLogESocial(' > getEnviosParaEvento elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap));
            /*- LOG*/

            /*LOG -*/
            $lap = microtime(true);
            /*- LOG*/
            $this->eSocialService->setDadosTransmissao($nrEstruturam);
            /*LOG -*/
            $this->eSocialService->saveLogESocial(' > setDadosTransmissao elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap));
            /*- LOG*/

            // Array que vai conter os lotes de eventos
            $eventosEnvio = array();
            $msgValidacao = array();
            
            // Array que armazenará as pendencias dos eventos
            $eventosNaoPeriodicosPendentes = array();

            /*LOG -*/
            $lap = microtime(true);
            /*- LOG*/
            try {
                $this->eSocialService->entityManager->beginTransaction();
                if($realizaVerificacao){
                    foreach($arrEnvios as $index => $envio) {
                        $anterior = $arrEnvios[$index - 1] ? $arrEnvios[$index - 1]->getNrchave() : null;
                        if($envio->getNrchave() != $anterior){
                            // fazer validação para verificar pendências nos envios de 1200 e 1210
                            $eventosPendentes = $this->eSocialService->getPendenciasEnvio($nmEvento, $envio->getNrchave(), $envio->getNrorg());
                            if(!empty($eventosPendentes)){
                                $eventosNaoPeriodicosPendentes = array_merge($eventosNaoPeriodicosPendentes, $eventosPendentes);
                                continue;
                            }
                        }   
                    }
                }
                if($realizaVerificacao && count($eventosNaoPeriodicosPendentes) == 0 || !$realizaVerificacao){
                    foreach($arrEnvios as $index => $envio) {
                        $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $envio->getNrenvio());
                        
                        $dadosEnvio = $envio->toArray();
                        $dadosEnvio['NMEVENTO'] = $nmEvento;
                        $dadosEnvio['IDGRUPO'] = $idGrupo;
                        $dadosEnvio['DTMESCOMPETENC'] = $dadosEnvio['DTMESCOMPETENC']->format('d/m/Y H:i:s');
                        if($dadosEnvio['IDSTATUS'] == 'E' || $dadosEnvio['IDSTATUS'] == 'R'){
                            $this->eSocialService->deleteDadosEnvio($dadosEnvio);
                        }
                        /*LOG -*/
                        $lap1 = microtime(true);
                        /*- LOG*/
                        $xmlEvento = $this->eSocialService->getXmlEvento($dsVersao, $dadosEnvio);
                        /*LOG -*/
                        $this->eSocialService->saveLogESocial(' > getXmlEvento elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap1));
                        /*- LOG*/
                        
                        if ($xmlEvento) {
                            $eventosEnvio[$envio->getNrenvio()] = $xmlEvento;
                        }
                        
                        foreach ($msgProcesso->getMessages() as $msg) {
                            $msgValidacao[] = array(
                                'NMEVENTO' => $dadosEnvio['NMEVENTO'],
                                'NRCHAVE' => $dadosEnvio['NRCHAVE'],
                                'MSG' => $msg['MSG'],
                                'TYPE' => $msg['TYPE'],
                            );
                        }
                        $msgProcesso->clear();
                        unset($msgProcesso);   
                    }
                }
                
            } catch (\Exception $e) {
                $this->eSocialService->entityManager->rollback();
                throw $e;
            }
            
            // verifica se foram identificadas pendências dentre os eventos
            // os eventos com pendências de eventos periódicos não são enviados
            if (!empty($eventosNaoPeriodicosPendentes) && $realizaVerificacao) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('pendenciasDeEnvioPeriodicos', $eventosNaoPeriodicosPendentes));
                $idOcorreuErro = false;
            }else{
                
                $this->eSocialService->entityManager->flush();
                $this->eSocialService->entityManager->commit();
                
                /*LOG -*/
                $this->eSocialService->saveLogESocial(' > foreach.getXmlEvento (' . count($arrEnvios) . ' envios) elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap), true);
                /*- LOG*/
        
                if (empty($eventosEnvio) && empty($msgValidacao) && empty($eventosNaoPeriodicosPendentes)) {
                    throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
                }
                

                // Verificar se ocorreu erro em alguns dos lotes
                $idOcorreuErro = false;
                $arrCodErros = array(
                    ESocialService::CD_RET_LOTE_301,
                    ESocialService::CD_RET_LOTE_401,
                    ESocialService::CD_RET_LOTE_402,
                    ESocialService::CD_RET_LOTE_403,
                    ESocialService::CD_RET_LOTE_404,
                    ESocialService::CD_RET_LOTE_405,
                );
    
                //Separa os eventos em grupos na quantidade máxima de eventos por lote
                $arrEventosEnvio = array_chunk($eventosEnvio, $this->qtdMaxEventosLote, true);
    
                /*LOG -*/
                $lap = microtime(true);
                /*- LOG*/
                //Envia os lotes com os eventos
                foreach ($arrEventosEnvio as $eventos) {
                    try {
                        //Transação a cada lote enviado para caso ocorra erro de conexão durante o envio de um dos lotes
                        $this->eSocialService->entityManager->beginTransaction();
                        
                        /*LOG -*/
                        $lap1 = microtime(true);
                        /*- LOG*/
                        $xmlLoteEmpregador = $this->eSocialService->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventos);
                        /*LOG -*/
                        $this->eSocialService->saveLogESocial(' >> getXmlLote elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap1));
                        $lap1 = microtime(true);
                        /*- LOG*/
                        $retornoLote = $this->eSocialService->enviarLote($eventos, $xmlLoteEmpregador);
                        /*LOG -*/
                        $this->eSocialService->saveLogESocial(' >> retornoLote [' . $retornoLote['COD'] . '-' . $retornoLote['MSG'] . ']');
                        $this->eSocialService->saveLogESocial(' >> enviarLote elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap1));
                        /*- LOG*/
                        if (array_search($retornoLote['COD'], $arrCodErros) != false) {
                            $idOcorreuErro = true;
                        }
                        
                        // inserir consulta por lote
                        
                        //Commit das alterações realizada nos envios do lote transmitido
                        $this->eSocialService->entityManager->flush();
                        $this->eSocialService->entityManager->commit();
                    } catch (\Exception $e) {
                        $this->eSocialService->entityManager->rollback();
                        $this->eSocialService->saveLogESocial(' > exception.getXmlLote/enviarLote [' . $e->getMessage() . ']');
                        throw $e;
                    }
                
                    //Delay para evitar erro de precedencia de envio de lotes em que um lote é enviado enquanto o anterior ainda está em processamento
                    sleep(5);
                }
                /*LOG -*/
                $this->eSocialService->saveLogESocial(' > foreach.getXmlLote/enviarLote (' . count($arrEventosEnvio) . ' lotes) elapsed ' . $this->eSocialService->convertSecondsToTime(microtime(true) - $lap));
                /*- LOG*/
    
//                $this->eSocialService->entityManager->flush();
//                $this->eSocialService->entityManager->commit();
                umask($oldmask);
                
            }
            
            if (count($msgValidacao) > 0) {
                 $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('setTransmissaoESocial', $msgValidacao));
            }
            if ($idOcorreuErro) {
                $response->addNotification(new DTO\Response\Notification('envioAutomaticoEventos', DTO\Response\Notification::TYPE_ALERT));
            } else {
                $response->addNotification(new DTO\Response\Notification('envioAutomaticoEventos', DTO\Response\Notification::TYPE_SUCCESS));
            }
            
            
            
            /*LOG -*/
            $timeFromStart = microtime(true) - $start;
            $this->eSocialService->saveLogESocial('=== envioAutomaticoEventos ends elapsed ' . $this->eSocialService->convertSecondsToTime($timeFromStart), true);
            /*- LOG*/
        } catch(\Exception $e) {
            /*LOG -*/
            $timeFromStart = microtime(true) - $start;
            $this->eSocialService->saveLogESocial('=== envioAutomaticoEventos ends elapsed ' . $this->eSocialService->convertSecondsToTime($timeFromStart) . "(" . $e->getMessage() . ")", true);
            /*- LOG*/
//            $this->eSocialService->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function consultaAutomaticaEventos(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $oldmask = umask(0);

            $filter = current($request->getDataSet()->getRows());

            $nrEstruturam = $filter['NRESTRUTURAM'];
            $nrEvento = $filter['NREVENTO'];
            $nmEvento = $filter['NMEVENTO'];
            $idGrupo = $filter['IDGRUPO'];
            $idStatus = $filter['IDSTATUS'];
            $dtmescompetenc = $this->hcmParameters->getDtcompetencia();

            $paramESocial = $this->eSocialService->getParamESocial($nrEstruturam);

            // Verifica se a estrutura tem parâmetro do eSocial
            if (empty($paramESocial)) {
                throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR);
            }

            $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
            $nrtpambiente = $paramESocial['NRTPAMBIENTE'];
            $nrtabela = array_key_exists('NRTIPOMOVIMENT', $filter) ? $filter['NRTIPOMOVIMENT'] : null;
            $nrtabela = !empty($nrtabela) ? $nrtabela : null;
            $nmEvento = $filter['NMEVENTO'];
            // Busca os itens pendentes que serão enviados por evento
            $arrEnvios = $this->eSocialService->getEnviosParaEvento($nrEstruturam, $nrtpambiente, $nrEvento, $idStatus, $dtmescompetenc, $idGrupo, $nmEvento, $nrtabela);
            if (empty($arrEnvios)) {
                throw new \Exception(Messages::HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR);
            }

            // Armazena os diferentes recibos existentes nos envios filtrados
            $arrRecibos = array();
            foreach ($arrEnvios as $envio) {
                if (!isset($arrRecibos[$envio->getNrrecibo()])) {
                    $arrRecibos[$envio->getNrrecibo()] = $envio->getNrrecibo();
                }
            }

            $this->eSocialService->setDadosTransmissao($nrEstruturam);

            sort($arrRecibos);
            foreach ($arrRecibos as $nrRecibo) {
                try {
                    $this->eSocialService->entityManager->beginTransaction();
                    
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                    $retorno = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                    $msgRetorno = $retorno['MSG'];
                    
                    $this->eSocialService->entityManager->flush();
                    $this->eSocialService->entityManager->commit();
                } catch(\Exception $e) { 
                    $this->eSocialService->entityManager->rollback();
                    throw $e;
                }
            }
            umask($oldmask);

            $response->addNotification(new DTO\Response\Notification('consultaAutomaticaEventos', DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function fechamentoPeriodico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            $msgValidacao = array();

            $row = current($request->getDataSet()->getRows());

            $this->eSocialService->entityManager->beginTransaction();

            $paramESocial = $this->eSocialService->getParamESocial($row['NRESTRUTURAM']);

            // Verifica se a estrutura tem parâmetro do eSocial
            if (empty($paramESocial)) {
                throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR);
            }
            
            $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRESTRUTURAM']);
            $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
            $idGrupo = 'P';
            
            $envio = $this->eSocialService->setEnvioFechamento($row['NRORG'], $row['NRESTRUTURAM'], $paramESocial['DSVERSAO'], $paramESocial['NRTPAMBIENTE'], $row['NRTABELA'], $row['SMOVIMENTO']);
            
            foreach ($msgProcesso->getMessages() as $msg) {
                $msgValidacao[] = array(
                    'NMESTRUTURAM' => $row['NMESTRUTURAM'],
                    'NRESTRUTURAM' => $row['NRESTRUTURAM'],
                    'MSG' => $msg['MSG'],
                    'TYPE' => $msg['TYPE'],
                );
            }
            
            $msgProcesso->clear();
            unset($msgProcesso);
            
            if (empty($msgValidacao)) {
                
                $arrayEnvio = $envio->toArray();
                $arrayEnvio['NMEVENTO'] = 'S-1299'; 
                $arrayEnvio['IDGRUPO'] = $idGrupo;
                $arrayEnvio['DTMESCOMPETENC'] = $arrayEnvio['DTMESCOMPETENC']->format('d/m/Y');
            
                $this->eSocialService->setDadosTransmissao($arrayEnvio['NRESTRUTURAM']);
                
                $xmlEvento = $this->eSocialService->getXmlEvento($dsVersao, $arrayEnvio);
                
                if ($xmlEvento) {
                    $eventosLote[] = $xmlEvento;
                }
                
                if(empty($eventosLote)) {
                    throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
                }

                $xmlLoteEventos = $this->eSocialService->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventosLote);
                $retorno = $this->eSocialService->enviarLote($eventosLote, $xmlLoteEventos);
                
                $codRetorno = $retorno['COD'];
                $nrRecibo = $retorno['REC'];
                $msgRetorno = $retorno['MSG'];
                
                // Caso o evento de exclusão/retificação seja enviado com sucesso realiza a consulta automática
                if ($codRetorno == '201') {
                    $this->eSocialService->entityManager->flush();
                    $this->eSocialService->entityManager->commit();
                    $this->eSocialService->entityManager->beginTransaction();
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                    $retornoConsulta = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                    $codRetornoConsulta = $retornoConsulta['COD'];
                    $msgRetornoConsulta = $retornoConsulta['MSG'];
                    $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    if ($codRetornoConsulta == '101') {
                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                        sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                        $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                        $retornoConsulta = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                        $codRetornoConsulta = $retornoConsulta['COD'];
                        $msgRetornoConsulta = $retornoConsulta['MSG'];
                        $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    }
                    $msgValidacao = $codRetornoConsulta . '-' . $msgRetornoConsulta . ($tmpEsperaConsulta ? '(Tempo de Espera:'.$tmpEsperaConsulta.')' : '');
                }else{
                    $msgValidacao = $codRetorno . '-' . $msgRetorno;
                }
                
                $this->eSocialService->entityManager->flush();
                $this->eSocialService->entityManager->commit();
                $response->addMessage(new DTO\Response\Message($msgValidacao));
                $response->addNotification(new DTO\Response\Notification('fechamentoPeriodico', DTO\Response\Notification::TYPE_SUCCESS));
            } else {
                $this->eSocialService->entityManager->rollback();
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('fechamentoPeriodico', $msgValidacao));
                $response->addNotification(new DTO\Response\Notification('fechamentoPeriodico', DTO\Response\Notification::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $this->eSocialService->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function reaberturaPeriodico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            $msgValidacao = array();

            $row = current($request->getDataSet()->getRows());
            
            $this->eSocialService->entityManager->beginTransaction();

            $paramESocial = $this->eSocialService->getParamESocial($row['NRESTRUTURAM']);

            // Verifica se a estrutura tem parâmetro do eSocial
            if (empty($paramESocial)) {
                throw new \Exception(Messages::HCM09000_ESO_PARAMETRO_NOT_FOUND_ERROR);
            }
            
            $msgProcesso = MsgProcesso::getInstance($this->hcmParameters->getNrorg(), $this->hcmParameters->getCdoperador(), $row['NRESTRUTURAM']);
            
            $dsVersao = str_replace('.','-',$paramESocial['DSVERSAO']);
            $idGrupo = 'P';
            
            $envio = $this->eSocialService->setEnvioReabertura($row['NRORG'], $row['NRESTRUTURAM'], $paramESocial['DSVERSAO'], $paramESocial['NRTPAMBIENTE'], $row['NRTABELA']);
            
            $arrayEnvio = $envio->toArray();
            $arrayEnvio['NMEVENTO'] = 'S-1298'; 
            $arrayEnvio['IDGRUPO'] = $idGrupo;
            $arrayEnvio['DTMESCOMPETENC'] = $arrayEnvio['DTMESCOMPETENC']->format('d/m/Y');
        
            $this->eSocialService->setDadosTransmissao($arrayEnvio['NRESTRUTURAM']);
            
            $xmlEvento = $this->eSocialService->getXmlEvento($dsVersao, $arrayEnvio);
            
            if ($xmlEvento) {
                $eventosLote[] = $xmlEvento;
            }
            
            foreach ($msgProcesso->getMessages() as $msg) {
                $msgValidacao[] = array(
                    'NMESTRUTURAM' => $row['NMESTRUTURAM'],
                    'NRESTRUTURAM' => $row['NRESTRUTURAM'],
                    'MSG' => $msg['MSG'],
                    'TYPE' => $msg['TYPE'],
                );
            }
            $msgProcesso->clear();
            unset($msgProcesso);
            
            if (empty($msgValidacao) && empty($eventosLote)) {
                throw new \Exception("HCM09000_ESO_LOTE_VAZIO_ERROR");
            }else if (empty($msgValidacao)) {

                $xmlLoteEventos = $this->eSocialService->getXmlLote($idGrupo, $paramESocial, $dsVersao, $eventosLote);
                $retorno = $this->eSocialService->enviarLote($eventosLote, $xmlLoteEventos);
                
                $codRetorno = $retorno['COD'];
                $nrRecibo = $retorno['REC'];
                $msgRetorno = $retorno['MSG'];
                
                // Caso o evento de exclusão/retificação seja enviado com sucesso realiza a consulta automática
                if ($codRetorno == '201') {
                    $this->eSocialService->entityManager->flush();
                    $this->eSocialService->entityManager->commit();
                    $this->eSocialService->entityManager->beginTransaction();
                    $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                    $retornoConsulta = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                    $codRetornoConsulta = $retornoConsulta['COD'];
                    $msgRetornoConsulta = $retornoConsulta['MSG'];
                    $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    if ($codRetornoConsulta == '101') {
                        // Caso o evento ainda esteja em processamento aguarda o tempo indicado e faz nova consulta
                        sleep($tmpEsperaConsulta ? (int)$tmpEsperaConsulta : 3);
                        $xmlConsulta = $this->eSocialService->getXmlConsulta($nrRecibo, $dsVersao);
                        $retornoConsulta = $this->eSocialService->consultarEvento($nrRecibo, $xmlConsulta);
                        $codRetornoConsulta = $retornoConsulta['COD'];
                        $msgRetornoConsulta = $retornoConsulta['MSG'];
                        $tmpEsperaConsulta = $retornoConsulta['TMP'];
                    }
                    $msgValidacao = $codRetornoConsulta . '-' . $msgRetornoConsulta . ($tmpEsperaConsulta ? '(Tempo de Espera:'.$tmpEsperaConsulta.')' : '');
                }else{
                    $msgValidacao = $codRetorno . '-' . $msgRetorno;
                }
                
                
                $this->eSocialService->entityManager->flush();
                $this->eSocialService->entityManager->commit();
                $response->addMessage(new DTO\Response\Message($msgValidacao));
                $response->addNotification(new DTO\Response\Notification('reaberturaPeriodico', DTO\Response\Notification::TYPE_SUCCESS));
            } else {
                $this->eSocialService->entityManager->rollback();
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('reaberturaPeriodico', $msgValidacao));
                $response->addNotification(new DTO\Response\Notification('reaberturaPeriodico', DTO\Response\Notification::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $this->eSocialService->entityManager->rollback();
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    protected function processEnvio($chave, $nomeChave, $row, $request, $response) {
        $row[$nomeChave] = $chave;
        $row['IDATIVO'] = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
        $row['DSDIRXMLENV'] = isset($row['DSDIRXMLENV']) ? $row['DSDIRXMLENV'] : null;
        $row['DSDIRXMLRETENV'] = isset($row['DSDIRXMLRETENV']) ? $row['DSDIRXMLRETENV'] : null;
        $row['DSDIRXMLRETPRO'] = isset($row['DSDIRXMLRETPRO']) ? $row['DSDIRXMLRETPRO'] : null;
        $row['DSDIRXMLRETTOT'] = isset($row['DSDIRXMLRETTOT']) ? $row['DSDIRXMLRETTOT'] : null;
        if ($row['NRTPAMBIENTE'] == '') {
            $paramESocial = $this->eSocialService->getParamESocial($row['NRESTRUTURAM']);
            if ($paramESocial) {
                $row['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
            }
        }
        
        /*if($row['NMEVENTO'] == 'S-1250'){
            $arrItens[] = ['NRCHAVE' => $chave,
                           'NRTABELA' => '1',
                           'IDOPERACAO' => 'I',
                           'DTMESCOMPETENC' => $this->hcmParameters->getDtCompetencia()];
        }else{*/
            $arrItens = $this->eSocialService->getItensParaEnvioEvento($row['NMEVENTO'], $row['IDGRUPO'], $row);
        //}
        if (empty($arrItens)) {
            $this->addMensagemProcessEnvio(Messages::HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR, $row['NMEVENTO']);
            return;
        }
        
        foreach($arrItens as $item) {
            $row['NRCHAVE'] = $item['NRCHAVE'];
            $row['NRTABELA'] = isset($item['NRTABELA']) ? $item['NRTABELA'] : '';
            $row['IDOPERACAO'] = isset($item['IDOPERACAO']) ? $item['IDOPERACAO'] : 'I';
            $row['DTMESCOMPETENC'] = $item['DTMESCOMPETENC'];
            $row['NRVINCULOM'] = array_key_exists('NRVINCULOM', $item) ? $item['NRVINCULOM'] : null;
            
            /*if($row['NMEVENTO'] == 'S-1250'){
                $row['IDSTATUS'] = 'C';
                $row['DSOBSERVACAO'] = 'Sucesso.';
            }*/


            try {
                $this->eSocialService->beforeInsertEnvioCadastros($row['NRESTRUTURAM'], $row['NRTPAMBIENTE'], $row['NREVENTO'], $row['NRCHAVE'], $row['NRTABELA'], $row['IDOPERACAO'], $row['DTMESCOMPETENC']);
            } catch(\Exception $e) {
                if ($e->getCode() == Messages::ALERT_MESSAGE_CODE) {
                    if($e->getMessage() == Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR){
                        $this->addMensagemProcessEnvio(Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR, $row['NMEVENTO']);
                    } else {    
                        $response->addMessage(new DTO\Response\Message($e->getMessage()), DTO\Response\Message::TYPE_WARNING);
                    }
                    continue;
                } else {
                    throw $e;
                }
            }
            if (array_key_exists('NRENVIO', $row)) {
                if ($row['NRENVIO'] === '') {
                    $row['NRENVIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                }
            } else {
                $row['NRENVIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
            }
            parent::save($request, $response);
            if($row['GERADEMONSTENVIO'] == 'S'){
                $dsVersao = str_replace('.','-',$row['DSVERSAO']);
                $this->eSocialService->setDadosTransmissao($row['NRESTRUTURAM']);
                $xmlEvento = $this->eSocialService->getXmlEvento($dsVersao, $row);
                if($xmlEvento){
                    $arrayEnvio = $this->eSocialService->setXmlFileToDataSet($xmlEvento['XMLEVENTO']);
                    $this->eSocialService->setDadosEnvio($arrayEnvio, $xmlEvento['NRENVIO']);
                }
            }
            unset($row['NRENVIO']);
        }
        return true;
    }
    
    public function geraMatriculaESocial(DTO\Request\DataSet $request, DTO\Response $response){
        try {    
            $row = current($request->getDataSet()->getRows());
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrestruturam = $row['NRESTRUTLEGAL'];
            
            $retorno = $this->eSocialService->geraMatriculaESocial($nrorg, $nrestruturam);
            
            if($retorno){
                $response->addDataset(new DataSet('geraMatriculaESocial', $retorno));
            }
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }   

    }
    
    public function ativaEvento(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->eSocialService->entityManager->beginTransaction();
            $row = current($request->getDataSet()->getRows());
            
            $nrenvio = $row['NRENVIO'];
            $nrorg = $row['NRORG'];
            $nrestruturam = $row['NRESTRUTURAM'];
            
            $this->eSocialService->ativaEvento($nrenvio, $nrorg, $nrestruturam);
            
            $this->eSocialService->entityManager->flush();
            $this->eSocialService->entityManager->commit();

            $response->addNotification(new DTO\Response\Notification('o evento foi ativado com sucesso!', DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $this->eSocialService->entityManager->rollback();
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } 
    }
    
    public function getArquivoXML(DTO\Request\Dataset $request, DTO\Response $response){
        //Função que tem como objetivo verificar a existência dos arquivos XML gerados pelo sistema HCM retornar os mesmos para o frontend em caso afirmativo
        //Função recebe um array de eventos, trata e retorna cada evento individualmente. Caso o arquivo exista e esteja disponível a url do download é retornado e, caso algum erro ocorra, dados do 
        //acontecido são rerornados
        
        $arquivos = [];
        
        foreach ($request->getDataSet()->getRows()[0] as $row){
            try{
                $nrorg    = $row['NRORG'];
                $xmlFile  = $row['IDXMLFILE'];
                $idEvento = $row['IDEVENTO'];
                $dsDirXml = $row['DSDIRXML'];
                $dsDirXmlFilesPath = $row['DSDIRXMLFILESPATH'];
                
                $nrCpfPessoa = isset($row['NRCPFPESSOA']) ? $row['NRCPFPESSOA'] : $row['NRCHAVE'];
                $nmEvento = $row['NMEVENTO'];
                $nrChave = $row['NRCHAVE'];
                
                $xml = $this->eSocialService->getXmlFiles($nrorg, $xmlFile, $idEvento, $dsDirXml, $dsDirXmlFilesPath);
                //$xml = $this->eSocialService->getXmlOnFilesPath($nrorg, $xmlFile, $idEvento, $dsDirXml);
                
                $arquivoXML = array('NMARQUIVO' => self::montaArquivo('XML_' . $xmlFile . '_' . $nmEvento .  '_' . $nrChave. '_' . $nrCpfPessoa . '_' . $idEvento . '.xml', $xml));
                
                $arquivos[] = $arquivoXML;
                
            } catch(\Exception $e){
                if(!$idEvento){
                    $idEvento = 'Não encontrado (status Pendente de Envio)';
                }
                $arquivos[] = array('ERRO' => '[EVENTO: '. $nmEvento . ', Chave: '. $nrChave . ', ID: '. $idEvento . ']: '.$e->getMessage() . '<br /><br />');
            }
        }
        
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivos', $arquivos));
    }
    
    public function montaArquivo($nmarquivo, $conteudo){
        $filePath = $this->hcmParameters->getTempDirectory().DIRECTORY_SEPARATOR.$nmarquivo;
        file_put_contents($filePath, $conteudo);
        return $filePath;
    }
    
    public function verificaVigenciaEnvioEsocial(DTO\Request\Dataset $request, DTO\Response $response){
        try{
            foreach ($request->getDataSet()->getRows()[0] as $row){
                $this->eSocialService->verificaVigenciaEnvioEsocial();
            }
        }catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } 
    }
    
    public function esvaziaMensagensProcessEnvio () {
        $this->mensagensProcessEnvio = array();
    }
    
    public function addMensagemProcessEnvio ($mensagem, $valor) {
        $mensagensExistentesArray = array_keys($this->mensagensProcessEnvio);
        
        if(!in_array($mensagem, $mensagensExistentesArray)){
            $this->mensagensProcessEnvio[$mensagem] = array($valor);
        } else {
            if(!in_array($valor, $this->mensagensProcessEnvio[$mensagem])){
                array_push($this->mensagensProcessEnvio[$mensagem], $valor);
            }
        }
    }
    
    public function existemMensagensProcessEnvio ($mensagem = null) {
        if (!empty($mensagem)) {
            return array_key_exists($mensagem, $this->mensagensProcessEnvio) ? !empty($this->mensagensProcessEnvio[$mensagem]) : false;
        } else {
            return count($this->mensagensProcessEnvio) > 0;
        }
    }
    
    /**
     * Retorna as mensagens salvas no array 'mensagensProcessEnvio' da seguinte maneira: mensagem1. (valor1, valor2. ...). mensagem2. (valor1, valor2. ...)
     */
    public function getMensagensProcessEnvio () {
        if ($this->existemMensagensProcessEnvio()) {
            $mensagemFinal = "";
            foreach ($this->mensagensProcessEnvio as $chave => $mensagens) {
                $mensagemFinal .= $chave;
                if(is_array($mensagens)){
                    $mensagemValores = "(";
                    foreach ($mensagens as $valor) {
                        $mensagemValores .= $valor . ", ";
                    }
                    $mensagemValores = substr($mensagemValores, 0 , -2);
                    $mensagemValores .= ")";
                    $mensagemFinal .= " " . $mensagemValores;
                }
                $mensagemFinal .= ". ";
            }
            return $mensagemFinal;
        } else {
            return false;
        }
    }
    
    public function setEnvio2200ManualAdmissao(DTO\Request\Dataset $request, DTO\Response $response){
        try{
            $dados = $request->getDataSet()->getRows()[0];
            $arrayEnvio = array();
            
            foreach ($dados['NRVINCULOM'] as $index => $vinculo) { 
                $paramESocial = $this->eSocialService->getParamESocial($dados['NRESTRUTURAM']);
                if ($paramESocial) {
                    $row['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
                }else{
                    $row['NRTPAMBIENTE'] = 1;
                }
                
                $row['NRCHAVE'] = $vinculo;
                $row['NRTABELA'] = '';
                $row['DSVERSAO'] = $dados['DSVERSAO'];
                $row['IDOPERACAO'] = 'I';
                if($dados['NRTIPOVINCULOM'][$index] == 1) {
                    $row['NREVENTO'] = 'S-2200';
                } else {
                    $row['NREVENTO'] = 'S-2300';
                }
                $row['NRVINCULOM'] = $vinculo;

                $ultimoHist = $this->eSocialService->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($this->hcmParameters->getNrorgTrabalho(), $vinculo, $this->hcmParameters->getDtcompetencia());
            	
                $row['NRESTRUTURAM'] = $dados['NRESTRUTURAM'];
                $row['DTMESCOMPETENC'] = date_format($ultimoHist->getDtmescompetenc(), 'd/m/Y H:i:s');
                $row['IDSTATUS'] = 'C';
                $row['DSOBSERVACAO'] = 'Inserido manualmente.';
                $row['IDATIVO'] = 'S';
                $row['DSDIRXMLENV'] = null;
                $row['DSDIRXMLRETENV'] = null;
                $row['DSDIRXMLRETPRO'] = null;
                $row['DSDIRXMLRETTOT'] = null;
                
                array_push($arrayEnvio, $row);
            }
            $this->eSocialService->setEnvio2200ManualAdmissao($arrayEnvio);
            
            
            $response->addMessage(new DTO\Response\Message('Envios inseridos com sucesso.', DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));	 
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}