<?php
namespace HCM\PTO\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\{ DateUtil, Repositories };
use HCM\PTO\Services\MarcacaoPontoService;

use Teknisa\Model\Gestaopessoas\Entities\GpeIntervaloshorario;

class HorarioRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }

    /**
     * @return array
     */
    public function removeSaidaHorario($dataset) {
        $query = $this->entityManager->getConnection()->prepare("
               UPDATE GPE_INTERVALOSHORARIO
                  SET HRPROJETARINTERVALO = null, DSINTERVALOPROJETADO = null
                WHERE NRORG              = :NRORG
                  AND NRHORDIAM          = :NRHORDIAM
                  AND NRINTERVALOHORARIO = :NRINTERVALOHORARIO
                  AND NRSEQINTERVHORARIO = :NRSEQINTERVHORARIO");

        $return = $query->execute(
            array(
                "NRORG"               => $dataset["NRORG"],
                "NRHORDIAM"           => $dataset["NRHORDIAM"],
                "NRSEQINTERVHORARIO"  => $dataset["NRSEQINTERVHORARIO"],
                "NRINTERVALOHORARIO"  => $dataset["NRINTERVALOHORARIO"]
            )
        );
        return $return;
    }
    
    /**
     *  Rotina que verifica se os intervalos de horário não incluem marcações 'DA' ou 'FA' que remetem à dias anteriores
     * 
     *  @param      $nrorg
     *  @param      $nrhordiam
     * 
     *  @return     Array   array('error' => $errorMesage, 'response' => boolean)
     */
    public function horarioIncluiDiaAnterior($nrorg, $nrhordiam): bool {
        $intervalosHorario = $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(array('nrorg' => $nrorg, 'nrhordiam' => $nrhordiam));;
        if (empty($intervalosHorario)) {
            return false;
        }
        
        foreach($intervalosHorario as $intervaloHorario) {
            if (in_array($intervaloHorario->getIddiaocorreentrada(), MarcacaoPontoService::IDDIAOCORRENCIA_ANTERIORES) || in_array($intervaloHorario->getIddiaocorresaida(), MarcacaoPontoService::IDDIAOCORRENCIA_ANTERIORES)) {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     *  Rotina que verifica se os intervalos de horário não incluem marcações 'DS' ou 'FS' que remetem à dias seguintes
     * 
     *  @param      $nrorg
     *  @param      $nrhordiam
     * 
     *  @return     Array   array('error' => $errorMesage, 'response' => boolean)
     */
    public function horarioIncluiDiaSeguinte($nrorg, $nrhordiam): bool {
        $intervalosHorario = $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(array('nrorg' => $nrorg, 'nrhordiam' => $nrhordiam));
        if (empty($intervalosHorario)) {
            return false;
        }
        
        foreach($intervalosHorario as $intervaloHorario) {
            if (in_array($intervaloHorario->getIddiaocorreentrada(), MarcacaoPontoService::IDDIAOCORRENCIA_SEGUINTES) || in_array($intervaloHorario->getIddiaocorresaida(), MarcacaoPontoService::IDDIAOCORRENCIA_SEGUINTES)) {
                return true;
            }
        }
        
        return false;
    }
    
    public function getIntervalosHorario(string $nrOrg, string $nrHordiam): ?Array {
        return $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(
            array('nrorg' => $nrOrg, 'nrhordiam' => $nrHordiam),
            array('nrseqintervhorario' => 'ASC')
        );
    }
    
    public function getUltimoIntervaloHorario(string $nrOrg, string $nrHordiam): ?GpeIntervaloshorario {
        $intervalosHorario = $this->getIntervalosHorario($nrOrg, $nrHordiam);
        return count($intervalosHorario) > 0 ? end($intervalosHorario) : null;
    }
    
    public function getPrimeiroIntervaloHorario(string $nrOrg, string $nrHordiam): ?GpeIntervaloshorario {
        $intervalosHorario = $this->getIntervalosHorario($nrOrg, $nrHordiam);
        return count($intervalosHorario) > 0 ? reset($intervalosHorario) : null;
    }

    public function getHoraUltimaMarcacaoHorario(string $nrOrg, string $nrHordiam): ?float {
        $ultimoIntervalo = $this->getUltimoIntervaloHorario($nrOrg, $nrHordiam);
        return !empty($ultimoIntervalo) ? DateUtil::getHoraFloatDeString($ultimoIntervalo->getHrsaida()) : null;
    }
    
    public function getHoraPrimeiraMarcacaoHorario(string $nrOrg, string $nrHordiam): ?float {
        $primeiroIntervalo = $this->getPrimeiroIntervaloHorario($nrOrg, $nrHordiam);
        return !empty($primeiroIntervalo) ? DateUtil::getHoraFloatDeString($primeiroIntervalo->getHrentrada()) : null;
    }
} 

