-- ============================================================
--   Database name:  FOLHA                                   
--   DBMS name:      ORACLE Version 12.x 
--   Created on:     06/08/2021  18:09                         
-- ============================================================

-- ============================================================
--   Type: EVOLMENSBH_TYPE                                      
-- ============================================================

CREATE OR REPLACE TYPE "EVOLMENSBH_TYPE" AS
OBJECT (
    NRVINCULOM              NUMBER,
    NMVINCULOM              VARCHAR2(100),
    DTADMISSAOVINC          DATE,
    DTRESCISAOVINC          DATE,
    CDMATRICULA             VARCHAR2(30),
    NRESTRUTGEREN           NUMBER,
    NMESTRUTGEREN           VARCHAR2(100),
    NRPERIODOAPURACAO       NUMBER,
    DSPERIODOAPURACAO       VARCHAR2(100),
    TOTAL_CREDITO           NUMBER,
    TOTAL_DEBITO            NUMBER,
    SALDO_BH                NUMBER,
    SALDO_BH_ANT            NUMBER,
    SALDO_QUITADO           NUMBER
)
/

-- ============================================================
--   Type: EVOLMENSBH_TABLE                                      
-- ============================================================
CREATE OR REPLACE TYPE "EVOLMENSBH_TABLE" AS TABLE OF EVOLMENSBH_TYPE
/

-- ============================================================
--   Function: EVOLMENSBH_DADOS                                      
-- ============================================================
CREATE OR REPLACE FUNCTION EVOLMENSBH_DADOS (
    P_NRORG             IN NUMBER,
    P_NRPERIODOAPURACAO IN NUMBER,
    P_DTMESCOMPETENC    IN DATE,
    P_NRVINCULOM        IN NUMBER,
    P_TPVINCULO         IN NUMBER,
    P_NRESTRUTLEGAL     IN NUMBER,
    P_NRESTRUTGEREN     IN NUMBER
)
RETURN EVOLMENSBH_TABLE PIPELINED
IS
  OUT_RECORD EVOLMENSBH_TYPE := EVOLMENSBH_TYPE (NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
BEGIN
    FOR DADOS_PRINCIPAIS IN (
        /* SELECT BASE PARA A CONSULTA DAS INFORMCAOES */
        WITH CALCULOS_PERIODO AS (
            SELECT
                VM.NRORG,
                VM.NRVINCULOM,
                VM.NMVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.CDMATRICULA,
                VH.DTMESCOMPETENC,
                VH.NRESTRUTGEREN,
                EM_GEREN.NMESTRUTURAM AS NMESTRUTGEREN,
                PA.NRPERIODOAPURACAO,
                TO_CHAR(PA.DTINICIALAPURACAO, 'DD/MM/YYYY') || ' - ' || TO_CHAR(PA.DTFINALAPURACAO, 'DD/MM/YYYY') || '(' || PA.IDPERIODOAPURACAO || ')' AS DSPERIODOAPURACAO,
                PA.DTINICIALAPURACAO,
                PA.DTFINALAPURACAO,
                RSF.NRRESUMOSITUFREQ,
                RSF.DTAPURACAOSITUFREQ,
                RSF.NRSITUFREQUENCIA,
                CALC.NRCALCULOSITUFREQ,
                RSF.QTREFAPURSITUFREQ
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= P_DTMESCOMPETENC
            )
            JOIN ESTRUTURAM EM_GEREN ON EM_GEREN.NRORG = VM.NRORG AND EM_GEREN.NRESTRUTURAM = VH.NRESTRUTGEREN
            
            /* BUSCA O PERIDO ATUAL E O ANTERIOR */
            JOIN PTO_PERIODOAPURACAO PA ON PA.NRORG = VM.NRORG AND PA.NRPERIODOAPURACAO IN (
                SELECT NRPERIODOAPURACAO
                FROM (
                    SELECT
                        PA.NRPERIODOAPURACAO        AS NRPERIODOAPURACAO_ATUAL,
                        PA_ANT.NRPERIODOAPURACAO    AS NRPERIODOAPURACAO_ANTERIOR
                    FROM PTO_PERIODOAPURACAO PA
                    LEFT JOIN PTO_PERIODOAPURACAO PA_ANT ON PA_ANT.NRORG = PA.NRORG AND PA_ANT.NRTIPOPERIODOAPURACAO = PA.NRTIPOPERIODOAPURACAO AND PA_ANT.DTINICIALAPURACAO = (
                        SELECT MAX(DTINICIALAPURACAO)
                        FROM PTO_PERIODOAPURACAO
                        WHERE NRORG = PA_ANT.NRORG
                            AND NRTIPOPERIODOAPURACAO = PA_ANT.NRTIPOPERIODOAPURACAO
                            /* CONDICOES DO PERIODO ANTERIOR */
                            AND DTINICIALAPURACAO < PA.DTINICIALAPURACAO
                            AND DTFINALAPURACAO < PA.DTINICIALAPURACAO
                    
                    )
                    WHERE PA.NRORG = P_NRORG AND PA.NRPERIODOAPURACAO = P_NRPERIODOAPURACAO
                )
                UNPIVOT (NRPERIODOAPURACAO FOR NRPERIODOAPURACAO_TIPO IN (NRPERIODOAPURACAO_ATUAL, NRPERIODOAPURACAO_ANTERIOR))
            )
        
            /* BUSCA OS RESUMOS (CALCULOS) GERADOS E A ASSOCIACAO DA SF GERADA COM O NRCALCULOSITUFREQ CALCULADO */
            JOIN PTO_RESUMOSITUFREQ RSF ON RSF.NRORG = VM.NRORG AND RSF.NRVINCULOM = VM.NRVINCULOM AND RSF.NRPERIODOAPURACAO = PA.NRPERIODOAPURACAO
            JOIN (
                SELECT
                    SF.NRSITUFREQUENCIA,
                    NVL(ASSOC_CLIENT.NRCALCULOSITUFREQ, ASSOC_PADRAO.NRCALCULOSITUFREQ) AS NRCALCULOSITUFREQ
                 FROM PTO_SITUFREQUENCIA SF
                 LEFT JOIN PTO_ASSOCSITUFREQ ASSOC_PADRAO ON ASSOC_PADRAO.NRORG = 0 AND ASSOC_PADRAO.NRSITUFREQUENCIA = SF.NRSITUFREQUENCIA
                 LEFT JOIN PTO_ASSOCSITUFREQ ASSOC_CLIENT ON ASSOC_CLIENT.NRORG = P_NRORG AND ASSOC_CLIENT.NRSITUFREQUENCIA = SF.NRSITUFREQUENCIA
                    /* FILTRO QUE BUSCA SOMENTE A PADRAO CASO EXISTAM VARIOS CALC ASSOCIADOS A UMA SITUFREQ DO CLIENTE */
                    AND (
                        (SF.NRORG = 0           AND ASSOC_CLIENT.NRSITUFREQUENCIA = ASSOC_CLIENT.NRCALCULOSITUFREQ)
                        OR (SF.NRORG <> 0)
                    )
                    /* SE EXISTIREM 2 CALCULOS ASSOCIADOS A UMS SITUFREQ, APENAS A PRIMEIRA EH CONSIDERADA */
                    AND NOT EXISTS (
                        SELECT 1
                        FROM PTO_ASSOCSITUFREQ
                        WHERE NRORG = P_NRORG
                            AND NRSITUFREQUENCIA = ASSOC_CLIENT.NRSITUFREQUENCIA
                            AND NRASSOCSITUFREQ <  ASSOC_CLIENT.NRASSOCSITUFREQ
                    )
                 WHERE
                    (SF.NRORG = P_NRORG OR SF.NRORG = 0)
            ) CALC ON CALC.NRSITUFREQUENCIA = RSF.NRSITUFREQUENCIA
            
            WHERE
                VM.NRORG = P_NRORG
                 AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC > LAST_DAY(PA.DTFINALAPURACAO))
                /* FILTROS DE VINCULOS */
                AND ( INSTR(',' || P_NRVINCULOM    || ',',     ',' || VM.NRVINCULOM        || ',') > 0 OR P_NRVINCULOM    IS NULL )
                AND ( INSTR(',' || P_TPVINCULO    || ',',      ',' || VM.NRTIPOVINCULOM    || ',') > 0 OR P_TPVINCULO     IS NULL )
                AND ( INSTR(',' || P_NRESTRUTLEGAL || ',',     ',' || VH.NRESTRUTLEGAL     || ',') > 0 OR P_NRESTRUTLEGAL IS NULL )
                AND ( INSTR(',' || P_NRESTRUTGEREN || ',',     ',' || VH.NRESTRUTGEREN     || ',') > 0 OR P_NRESTRUTGEREN IS NULL )
            ORDER BY VM.NRVINCULOM, RSF.DTAPURACAOSITUFREQ
        )
        
        /* SELECT PRINCIPAL */
        SELECT
            NRVINCULOM,
            NMVINCULOM,
            DTADMISSAOVINC,
            DTRESCISAOVINC,
            CDMATRICULA,
            NRESTRUTGEREN,
            NMESTRUTGEREN,
            NRPERIODOAPURACAO,
            DSPERIODOAPURACAO,
            TOTAL_CREDITO,
            TOTAL_DEBITO,
            /* BUSCA O SALDO DE BANCO DE HORAS DO ULTIMO DIA DO PERIODO */
            (
                SELECT QTREFAPURSITUFREQ
                FROM CALCULOS_PERIODO SUB_CALCULO
                WHERE SUB_CALCULO.NRPERIODOAPURACAO = P_NRPERIODOAPURACAO AND SUB_CALCULO.NRVINCULOM = PRINCIPAL.NRVINCULOM AND SUB_CALCULO.NRCALCULOSITUFREQ = 18420 AND DTAPURACAOSITUFREQ = (
                    SELECT MAX(DTAPURACAOSITUFREQ) FROM CALCULOS_PERIODO WHERE NRPERIODOAPURACAO = SUB_CALCULO.NRPERIODOAPURACAO AND NRVINCULOM = SUB_CALCULO.NRVINCULOM AND NRCALCULOSITUFREQ = SUB_CALCULO.NRCALCULOSITUFREQ
                )
            ) AS SALDO_BH,
            /* BUSCA O SALDO DE BANCO DE HORAS DO ULTIMO DIA DO PERIODO ANTERIOR */
            (
                SELECT QTREFAPURSITUFREQ
                FROM CALCULOS_PERIODO SUB_CALCULO
                WHERE SUB_CALCULO.NRPERIODOAPURACAO <> P_NRPERIODOAPURACAO AND SUB_CALCULO.NRVINCULOM = PRINCIPAL.NRVINCULOM AND SUB_CALCULO.NRCALCULOSITUFREQ = 18420 AND DTAPURACAOSITUFREQ = (
                    SELECT MAX(DTAPURACAOSITUFREQ) FROM CALCULOS_PERIODO WHERE NRPERIODOAPURACAO = SUB_CALCULO.NRPERIODOAPURACAO AND NRVINCULOM = SUB_CALCULO.NRVINCULOM AND NRCALCULOSITUFREQ = SUB_CALCULO.NRCALCULOSITUFREQ
                )
            ) AS SALDO_BH_ANT,
            /* BUSCA O ÚLTIMO SALDO DE BANCO DE HORAS QUITADO DO PERIODO */
            (
                SELECT QTREFAPURSITUFREQ
                FROM CALCULOS_PERIODO SUB_CALCULO
                WHERE SUB_CALCULO.NRPERIODOAPURACAO = P_NRPERIODOAPURACAO AND SUB_CALCULO.NRVINCULOM = PRINCIPAL.NRVINCULOM
                    AND (
                        SUB_CALCULO.NRCALCULOSITUFREQ = 8000
                        /* BUSCA POR SITUACOES DE FREQUENCIA ASSOCIADAS A OCORRENCIA DE QUITACAO */
                        OR SUB_CALCULO.NRSITUFREQUENCIA IN (
                            SELECT NRSITUFREQUENCIA
                            FROM PTO_PROGOCORRENCIA
                            WHERE NRORG = SUB_CALCULO.NRORG AND NRVINCULOM = SUB_CALCULO.NRVINCULOM AND DTINICIOPROGOCOR <= SUB_CALCULO.DTFINALAPURACAO AND (DTFIMPROGOCOR >= SUB_CALCULO.DTINICIALAPURACAO OR DTFIMPROGOCOR IS NULL) AND NRTIPOPROGOCORRENCIA IN (7, 8, 9)
                        )
                    )
                    AND DTAPURACAOSITUFREQ = (
                        SELECT MAX(DTAPURACAOSITUFREQ) FROM CALCULOS_PERIODO WHERE NRPERIODOAPURACAO = SUB_CALCULO.NRPERIODOAPURACAO AND NRVINCULOM = SUB_CALCULO.NRVINCULOM AND (NRCALCULOSITUFREQ = SUB_CALCULO.NRCALCULOSITUFREQ OR NRSITUFREQUENCIA = SUB_CALCULO.NRSITUFREQUENCIA)
                    )
            ) AS SALDO_QUITADO
        FROM (
            SELECT
                NRVINCULOM,
                NMVINCULOM,
                DTADMISSAOVINC,
                DTRESCISAOVINC,
                CDMATRICULA,
                NRESTRUTGEREN,
                NMESTRUTGEREN,
                NRPERIODOAPURACAO,
                DSPERIODOAPURACAO,
                NRCALCULOSITUFREQ,
                QTREFAPURSITUFREQ
            FROM CALCULOS_PERIODO
            WHERE NRPERIODOAPURACAO = P_NRPERIODOAPURACAO
        ) 
        PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRCALCULOSITUFREQ IN (16100 TOTAL_CREDITO, 17000 TOTAL_DEBITO)) PRINCIPAL
        ORDER BY NMVINCULOM
    )
    LOOP
        /* PREAPRA ROW DA TABELA A SER GERADA COM CADA ROW OBTIDA ATRAVES DA CONSULTA */
        OUT_RECORD.NRVINCULOM           := DADOS_PRINCIPAIS.NRVINCULOM;
        OUT_RECORD.NMVINCULOM           := DADOS_PRINCIPAIS.NMVINCULOM;
        OUT_RECORD.DTADMISSAOVINC       := DADOS_PRINCIPAIS.DTADMISSAOVINC;
        OUT_RECORD.DTRESCISAOVINC       := DADOS_PRINCIPAIS.DTRESCISAOVINC;
        OUT_RECORD.CDMATRICULA          := DADOS_PRINCIPAIS.CDMATRICULA;
        OUT_RECORD.NRESTRUTGEREN        := DADOS_PRINCIPAIS.NRESTRUTGEREN;
        OUT_RECORD.NMESTRUTGEREN        := DADOS_PRINCIPAIS.NMESTRUTGEREN;
        OUT_RECORD.NRPERIODOAPURACAO    := DADOS_PRINCIPAIS.NRPERIODOAPURACAO;
        OUT_RECORD.DSPERIODOAPURACAO    := DADOS_PRINCIPAIS.DSPERIODOAPURACAO;
        OUT_RECORD.TOTAL_CREDITO        := DADOS_PRINCIPAIS.TOTAL_CREDITO;
        OUT_RECORD.TOTAL_DEBITO         := DADOS_PRINCIPAIS.TOTAL_DEBITO;
        OUT_RECORD.SALDO_BH             := DADOS_PRINCIPAIS.SALDO_BH;
        OUT_RECORD.SALDO_BH_ANT         := DADOS_PRINCIPAIS.SALDO_BH_ANT;
        OUT_RECORD.SALDO_QUITADO        := DADOS_PRINCIPAIS.SALDO_QUITADO;
        PIPE ROW (OUT_RECORD);
    END LOOP;
END ;
/