<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Repositories\GpeEsocialRegistro;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-2299 - Desligamento
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S2299
{

    /**
     * Retorna os dados que serao utilizados para a geracao do(s) arquivo(s) do evento S-2299, Desligamento
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo)
    {
        $S2299 = GpeEsocialRegistro::dataSetEsocial_S2299(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo,
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo
        );

        foreach ($S2299 as &$row) {
            $verbas = GpeEsocialRegistro::s2299_verbasResc($row['NRVINCULOM'], $inicioPeriodo, $fimPeriodo, $nrorg);
            $cont = 1;
            foreach ($verbas as &$item) {
                $row['VERBASRESC'][$cont] =
                    array(
                        "_EVTDESLIG_INFODESLIG_VERBASRESC_CODRUBR" => $item['CODRUBR'],
                        "_EVTDESLIG_INFODESLIG_VERBASRESC_QTDRUBR" => $item['QTDRUBR'],
                        "_EVTDESLIG_INFODESLIG_VERBASRESC_VRRUBR" => $item['VLRRUBR']);
                $cont++;
            }

            $recPgtos[] = array(
                '_EVTDESLIG_INFODESLIG_RECPGTOS_IDERECPGTO' => $row['IDERECPAGTO'],
                '_EVTDESLIG_INFODESLIG_RECPGTOS_VLRPGTO' => $row['VLRLIQUIDO']
            );

            $row['RECPGTOS'] = $recPgtos;
        }

        return $S2299;
    }

}